@@ -16,7 +16,7 @@
 #include <QQmlEngine>
 #include <QStyleHints>
 
-#include "units.h"
+#include "platform/units.h"
 
 class QmlComponentsPoolSingleton
 {
@@ -59,12 +59,12 @@ QmlComponentsPool *QmlComponentsPoolSingleton::instance(QQmlEngine *engine)
 QmlComponentsPool::QmlComponentsPool(QQmlEngine *engine)
     : QObject(engine)
 {
-    QQmlComponent *component = new QQmlComponent(engine, this);
+    QQmlComponent component(engine);
 
     /* clang-format off */
-    component->setData(QByteArrayLiteral(R"(
-import QtQuick 2.7
-import org.kde.kirigami 2.7 as Kirigami
+    component.setData(QByteArrayLiteral(R"(
+import QtQuick
+import org.kde.kirigami as Kirigami
 
 QtObject {
     readonly property Component leadingSeparator: Kirigami.Separator {
@@ -73,6 +73,10 @@ QtObject {
         // positioning trick to hide the very first separator
         visible: {
             const view = column.Kirigami.ColumnView.view;
+            if (!view || !view.separatorVisible) {
+                return false;
+            }
+
             return view && (LayoutMirroring.enabled
                 ? view.contentX + view.width > column.x + column.width
                 : view.contentX < column.x);
@@ -98,7 +102,7 @@ QtObject {
 )"), QUrl(QStringLiteral("columnview.cpp")));
     /* clang-format on */
 
-    m_instance = component->create();
+    m_instance = component.create();
     // qCWarning(KirigamiLog)<<component->errors();
     Q_ASSERT(m_instance);
     m_instance->setParent(this);
@@ -109,11 +113,11 @@ QtObject {
     m_trailingSeparatorComponent = m_instance->property("trailingSeparator").value<QQmlComponent *>();
     Q_ASSERT(m_trailingSeparatorComponent);
 
-    m_units = engine->singletonInstance<Kirigami::Units *>(qmlTypeId("org.kde.kirigami", 2, 0, "Units"));
+    m_units = engine->singletonInstance<Kirigami::Platform::Units *>("org.kde.kirigami.platform", "Units");
     Q_ASSERT(m_units);
 
-    connect(m_units, &Kirigami::Units::gridUnitChanged, this, &QmlComponentsPool::gridUnitChanged);
-    connect(m_units, &Kirigami::Units::longDurationChanged, this, &QmlComponentsPool::longDurationChanged);
+    connect(m_units, &Kirigami::Platform::Units::gridUnitChanged, this, &QmlComponentsPool::gridUnitChanged);
+    connect(m_units, &Kirigami::Platform::Units::longDurationChanged, this, &QmlComponentsPool::longDurationChanged);
 }
 
 QmlComponentsPool::~QmlComponentsPool()
@@ -576,17 +580,18 @@ void ContentItem::updateVisibleItems()
     for (auto *item : std::as_const(m_visibleItems)) {
         disconnect(item, &QObject::destroyed, this, nullptr);
     }
-    const QQuickItem *oldFirstVisibleItem = m_visibleItems.isEmpty() ? nullptr : qobject_cast<QQuickItem *>(m_visibleItems.first());
-    const QQuickItem *oldLastVisibleItem = m_visibleItems.isEmpty() ? nullptr : qobject_cast<QQuickItem *>(m_visibleItems.last());
+
+    const QQuickItem *oldLeadingVisibleItem = m_view->leadingVisibleItem();
+    const QQuickItem *oldTrailingVisibleItem = m_view->trailingVisibleItem();
 
     if (newItems != m_visibleItems) {
         m_visibleItems = newItems;
         Q_EMIT m_view->visibleItemsChanged();
-        if (!m_visibleItems.isEmpty() && m_visibleItems.first() != oldFirstVisibleItem) {
-            Q_EMIT m_view->firstVisibleItemChanged();
+        if (!m_visibleItems.isEmpty() && m_visibleItems.first() != oldLeadingVisibleItem) {
+            Q_EMIT m_view->leadingVisibleItemChanged();
         }
-        if (!m_visibleItems.isEmpty() && m_visibleItems.last() != oldLastVisibleItem) {
-            Q_EMIT m_view->lastVisibleItemChanged();
+        if (!m_visibleItems.isEmpty() && m_visibleItems.last() != oldTrailingVisibleItem) {
+            Q_EMIT m_view->trailingVisibleItemChanged();
         }
     }
 }
@@ -716,19 +721,11 @@ void ContentItem::itemChange(QQuickItem::ItemChange change, const QQuickItem::It
     QQuickItem::itemChange(change, value);
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-void ContentItem::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
-{
-    updateVisibleItems();
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
-}
-#else
 void ContentItem::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     updateVisibleItems();
     QQuickItem::geometryChange(newGeometry, oldGeometry);
 }
-#endif
 
 void ContentItem::syncItemsOrder()
 {
@@ -776,6 +773,8 @@ ColumnView::ColumnView(QQuickItem *parent)
 {
     // NOTE: this is to *not* trigger itemChange
     m_contentItem = new ContentItem(this);
+    // Prevent interactions outside of ColumnView bounds, and let it act as a viewport.
+    setClip(true);
     setAcceptedMouseButtons(Qt::LeftButton | Qt::BackButton | Qt::ForwardButton);
     setAcceptTouchEvents(false); // Relies on synthetized mouse events
     setFiltersChildMouseEvents(true);
@@ -880,7 +879,11 @@ void ColumnView::setCurrentIndex(int index)
         if (!m_mouseDown) {
             if (!contentsRect.contains(mappedCurrent)) {
                 m_contentItem->m_viewAnchorItem = m_currentItem;
-                m_contentItem->animateX(-m_currentItem->x() + m_contentItem->m_leftPinnedSpace);
+                if (qApp->layoutDirection() == Qt::RightToLeft) {
+                    m_contentItem->animateX(-m_currentItem->x() - m_currentItem->width() + width());
+                } else {
+                    m_contentItem->animateX(-m_currentItem->x() + m_contentItem->m_leftPinnedSpace);
+                }
             } else {
                 m_contentItem->snapToItem();
             }
@@ -901,7 +904,7 @@ QList<QObject *> ColumnView::visibleItems() const
     return m_contentItem->m_visibleItems;
 }
 
-QQuickItem *ColumnView::firstVisibleItem() const
+QQuickItem *ColumnView::leadingVisibleItem() const
 {
     if (m_contentItem->m_visibleItems.isEmpty()) {
         return nullptr;
@@ -910,7 +913,7 @@ QQuickItem *ColumnView::firstVisibleItem() const
     return qobject_cast<QQuickItem *>(m_contentItem->m_visibleItems.first());
 }
 
-QQuickItem *ColumnView::lastVisibleItem() const
+QQuickItem *ColumnView::trailingVisibleItem() const
 {
     if (m_contentItem->m_visibleItems.isEmpty()) {
         return nullptr;
@@ -991,31 +994,6 @@ void ColumnView::setSeparatorVisible(bool visible)
 
     m_separatorVisible = visible;
 
-    if (visible) {
-        for (QQuickItem *item : std::as_const(m_contentItem->m_items)) {
-            QQuickItem *sep = m_contentItem->ensureLeadingSeparator(item);
-            if (sep) {
-                sep->setVisible(true);
-            }
-
-            ColumnViewAttached *attached = qobject_cast<ColumnViewAttached *>(qmlAttachedPropertiesObject<ColumnView>(item, true));
-            if (attached->isPinned()) {
-                QQuickItem *sep = m_contentItem->ensureTrailingSeparator(item);
-                if (sep) {
-                    sep->setVisible(true);
-                }
-            }
-        }
-
-    } else {
-        for (QQuickItem *sep : std::as_const(m_contentItem->m_leadingSeparators)) {
-            sep->setVisible(false);
-        }
-        for (QQuickItem *sep : std::as_const(m_contentItem->m_trailingSeparators)) {
-            sep->setVisible(false);
-        }
-    }
-
     Q_EMIT separatorVisibleChanged();
 }
 
@@ -1216,17 +1194,6 @@ void ColumnView::moveItem(int from, int to)
     polish();
 }
 
-QQuickItem *ColumnView::removeItem(const QVariant &item)
-{
-    if (item.canConvert<QQuickItem *>()) {
-        return removeItem(item.value<QQuickItem *>());
-    } else if (item.canConvert<int>()) {
-        return removeItem(item.toInt());
-    } else {
-        return nullptr;
-    }
-}
-
 QQuickItem *ColumnView::removeItem(QQuickItem *item)
 {
     if (m_contentItem->m_items.isEmpty() || !m_contentItem->m_items.contains(item)) {
@@ -1257,35 +1224,73 @@ QQuickItem *ColumnView::removeItem(QQuickItem *item)
     return item;
 }
 
-QQuickItem *ColumnView::removeItem(int pos)
+QQuickItem *ColumnView::removeItem(const int index)
 {
-    if (m_contentItem->m_items.isEmpty() //
-        || pos < 0 || pos >= m_contentItem->m_items.length()) {
+    if (m_contentItem->m_items.isEmpty() || index < 0 || index >= count()) {
         return nullptr;
+    } else {
+        return removeItem(m_contentItem->m_items[index]);
     }
-
-    return removeItem(m_contentItem->m_items[pos]);
 }
 
+QQuickItem *ColumnView::removeItem(const QVariant &item)
+{
+    if (item.canConvert<QQuickItem *>()) {
+        return removeItem(item.value<QQuickItem *>());
+    } else if (item.canConvert<int>()) {
+        return removeItem(item.toInt());
+    } else {
+        return nullptr;
+    }
+}
+
+QQuickItem *ColumnView::pop(const QVariant &item)
+{
+    if (item.canConvert<QQuickItem *>()) {
+        return pop(item.value<QQuickItem *>());
+    } else if (item.canConvert<int>()) {
+        return pop(item.toInt());
+    } else if (item.isNull()) {
+        return pop();
+    }
+    return nullptr;
+}
 QQuickItem *ColumnView::pop(QQuickItem *item)
 {
     QQuickItem *removed = nullptr;
 
     while (!m_contentItem->m_items.isEmpty() && m_contentItem->m_items.last() != item) {
         removed = removeItem(m_contentItem->m_items.last());
-        // if no item has been passed, just pop one
-        if (!item) {
-            break;
-        }
     }
     return removed;
 }
 
+QQuickItem *ColumnView::pop(const int index)
+{
+    if (index >= 0 && index < count() - 1) {
+        return pop(m_contentItem->m_items.at(index));
+    } else if (index == -1) {
+        return pop(nullptr);
+    }
+    return nullptr;
+}
+
+QQuickItem *ColumnView::pop()
+{
+    if (count() > 0) {
+        return removeItem(count() - 1);
+    }
+    return nullptr;
+}
+
 void ColumnView::clear()
 {
-    for (QQuickItem *item : std::as_const(m_contentItem->m_items)) {
+    // Don't do an iterator on a list that gets progressively destroyed, treat it as a stack
+    while (!m_contentItem->m_items.isEmpty()) {
+        QQuickItem *item = m_contentItem->m_items.first();
         removeItem(item);
     }
+
     m_contentItem->m_items.clear();
     Q_EMIT contentChildrenChanged();
 }
@@ -1305,11 +1310,7 @@ ColumnViewAttached *ColumnView::qmlAttachedProperties(QObject *object)
     return new ColumnViewAttached(object);
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-void ColumnView::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
-#else
 void ColumnView::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
-#endif
 {
     m_contentItem->setY(m_topPadding);
     m_contentItem->setHeight(newGeometry.height() - m_topPadding - m_bottomPadding);
@@ -1317,11 +1318,7 @@ void ColumnView::geometryChange(const QRectF &newGeometry, const QRectF &oldGeom
     polish();
 
     m_contentItem->updateVisibleItems();
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
-#else
     QQuickItem::geometryChange(newGeometry, oldGeometry);
-#endif
 }
 
 bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
@@ -1358,8 +1355,8 @@ bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
             m_contentItem->snapToItem();
             return false;
         }
-        m_oldMouseX = m_startMouseX = mapFromItem(item, me->localPos()).x();
-        m_oldMouseY = m_startMouseY = mapFromItem(item, me->localPos()).y();
+        m_oldMouseX = m_startMouseX = mapFromItem(item, me->position()).x();
+        m_oldMouseY = m_startMouseY = mapFromItem(item, me->position()).y();
 
         m_mouseDown = true;
         me->setAccepted(false);
@@ -1378,7 +1375,7 @@ bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
             return false;
         }
 
-        const QPointF pos = mapFromItem(item, me->localPos());
+        const QPointF pos = mapFromItem(item, me->position());
 
         bool verticalScrollIntercepted = false;
 
@@ -1416,7 +1413,7 @@ bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
 
         const bool wasDragging = m_dragging;
         // If a drag happened, start to steal all events, use startDragDistance * 2 to give time to widgets to take the mouse grab by themselves
-        m_dragging = keepMouseGrab() || qAbs(mapFromItem(item, me->localPos()).x() - m_startMouseX) > qApp->styleHints()->startDragDistance() * 3;
+        m_dragging = keepMouseGrab() || qAbs(mapFromItem(item, me->position()).x() - m_startMouseX) > qApp->styleHints()->startDragDistance() * 3;
 
         if (m_dragging != wasDragging) {
             m_moving = true;
@@ -1503,8 +1500,8 @@ void ColumnView::mousePressEvent(QMouseEvent *event)
     }
 
     m_contentItem->snapToItem();
-    m_oldMouseX = event->localPos().x();
-    m_startMouseX = event->localPos().x();
+    m_oldMouseX = event->position().x();
+    m_startMouseX = event->position().x();
     m_mouseDown = true;
     setKeepMouseGrab(false);
     event->accept();
@@ -1523,7 +1520,7 @@ void ColumnView::mouseMoveEvent(QMouseEvent *event)
 
     const bool wasDragging = m_dragging;
     // Same startDragDistance * 2 as the event filter
-    m_dragging = keepMouseGrab() || qAbs(event->localPos().x() - m_startMouseX) > qApp->styleHints()->startDragDistance() * 2;
+    m_dragging = keepMouseGrab() || qAbs(event->position().x() - m_startMouseX) > qApp->styleHints()->startDragDistance() * 2;
     if (m_dragging != wasDragging) {
         m_moving = true;
         Q_EMIT movingChanged();
@@ -1654,11 +1651,7 @@ void ColumnView::contentChildren_append(QQmlListProperty<QQuickItem> *prop, QQui
     item->setParentItem(view->m_contentItem);
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-int ColumnView::contentChildren_count(QQmlListProperty<QQuickItem> *prop)
-#else
 qsizetype ColumnView::contentChildren_count(QQmlListProperty<QQuickItem> *prop)
-#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
@@ -1668,11 +1661,7 @@ qsizetype ColumnView::contentChildren_count(QQmlListProperty<QQuickItem> *prop)
     return view->m_contentItem->m_items.count();
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-QQuickItem *ColumnView::contentChildren_at(QQmlListProperty<QQuickItem> *prop, int index)
-#else
 QQuickItem *ColumnView::contentChildren_at(QQmlListProperty<QQuickItem> *prop, qsizetype index)
-#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
@@ -1737,11 +1726,7 @@ void ColumnView::contentData_append(QQmlListProperty<QObject> *prop, QObject *ob
     }
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-int ColumnView::contentData_count(QQmlListProperty<QObject> *prop)
-#else
 qsizetype ColumnView::contentData_count(QQmlListProperty<QObject> *prop)
-#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
@@ -1751,11 +1736,7 @@ qsizetype ColumnView::contentData_count(QQmlListProperty<QObject> *prop)
     return view->m_contentData.count();
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-QObject *ColumnView::contentData_at(QQmlListProperty<QObject> *prop, int index)
-#else
 QObject *ColumnView::contentData_at(QQmlListProperty<QObject> *prop, qsizetype index)
-#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
@@ -1789,3 +1770,4 @@ QQmlListProperty<QObject> ColumnView::contentData()
 }
 
 #include "moc_columnview.cpp"
+#include "moc_columnview_p.cpp"
