@@ -13,111 +13,87 @@
 class QQuickItem;
 
 /**
- * @brief This attached property contains the information for decorating a FormLayout:
+ * This attached property contains the information for decorating a org::kde::kirigami::FormLayout:
  *
  * It contains the text labels of fields and information about sections.
  *
- * Some of its properties can be used with other QtQuick.Layouts.Layout types.
- *
- * Example usage:
- * @include formlayout.qml
- *
+ * Some of its properties can be used with other <a href="https://doc.qt.io/qt-6/qml-qtquick-layouts-layout.html">Layout</a> types.
+ * @code
+ * import org.kde.kirigami 2.3 as Kirigami
+ * Kirigami.FormLayout {
+ *    TextField {
+ *       Kirigami.FormData.label: "Label:"
+ *    }
+ *    TextField {
+ *       Kirigami.FormData.label: "Label:"
+ *    }
+ * }
+ * @endcode
  * @see org::kde::kirigami::FormLayout
- * @since org.kde.kirigami 2.3
+ * @since 2.3
  */
 class FormLayoutAttached : public QObject
 {
     Q_OBJECT
     /**
-     * @brief This property holds the text for the field's label.
+     * The label for a org::kde::kirigami::FormLayout field
      */
-    Q_PROPERTY(QString label READ label WRITE setLabel NOTIFY labelChanged)
-
+    Q_PROPERTY(QString label READ label WRITE setLabel NOTIFY labelChanged FINAL)
     /**
-     * @brief This property holds the alignment for the field's label.
+     * The alignment for the label of a org::kde::kirigami::FormLayout field
      */
-    Q_PROPERTY(int labelAlignment READ labelAlignment WRITE setLabelAlignment NOTIFY labelAlignmentChanged)
-
+    Q_PROPERTY(int labelAlignment READ labelAlignment WRITE setLabelAlignment NOTIFY labelAlignmentChanged FINAL)
     /**
-     * @brief This property sets whether this field acts as a section separator.
-     *
-     * default: ``false``
-     *
-     * You can use it in the following ways:
-     * * As space between two fields:
+     * If true, the child item of a org::kde::kirigami::FormLayout becomes a section separator, and
+     * may have different looks:
+     * * To make it just a space between two fields, just put an empty item with FormData.isSection:
      * @code
-     * QQC2.TextField {
+     * TextField {
      *     Kirigami.FormData.label: "Label:"
      * }
      * Item {
      *     Kirigami.FormData.isSection: true
      * }
-     * QQC2.TextField {
+     * TextField {
      *     Kirigami.FormData.label: "Label:"
      * }
      * @endcode
      *
-     * * As space with a section title:
+     * * To make it a space with a section title:
      * @code
-     * QQC2.TextField {
+     * TextField {
      *     Kirigami.FormData.label: "Label:"
      * }
      * Item {
      *     Kirigami.FormData.label: "Section Title"
      *     Kirigami.FormData.isSection: true
      * }
-     * QQC2.TextField {
-     *     Kirigami.FormData.label: "Label text"
+     * TextField {
+     *     Kirigami.FormData.label: "Label:"
      * }
      * @endcode
      *
-     * * As space with a section title and a separator line:
+     * * To make it a space with a section title and a separator line:
      * @code
-     * QQC2.TextField {
+     * TextField {
      *     Kirigami.FormData.label: "Label:"
      * }
      * Kirigami.Separator {
      *     Kirigami.FormData.label: "Section Title"
      *     Kirigami.FormData.isSection: true
      * }
-     * QQC2.TextField {
+     * TextField {
      *     Kirigami.FormData.label: "Label:"
      * }
      * @endcode
-     *
      * @see org::kde::kirigami::FormLayout
      */
-    Q_PROPERTY(bool isSection READ isSection WRITE setIsSection NOTIFY isSectionChanged)
-
-    /**
-     * @brief This property sets whether a checkbox should be added before the field's item.
-     *
-     * default: ``false``
-     */
-    Q_PROPERTY(bool checkable READ checkable WRITE setCheckable NOTIFY checkableChanged)
-
-    /**
-     * @brief This property sets whether the checkbox created by the ::checkable
-     * property should be checked.
-     *
-     * default: ``false``
-     *
-     * @see ::checkable
-     */
-    Q_PROPERTY(bool checked READ checked WRITE setChecked NOTIFY checkedChanged)
-
-    /**
-     * @brief This property sets whether the label or checkbox created by the
-     * FormLayout attached property should be enabled.
-     *
-     * default: ``true``
-     */
-    Q_PROPERTY(bool enabled READ enabled WRITE setEnabled NOTIFY enabledChanged)
+    Q_PROPERTY(bool isSection READ isSection WRITE setIsSection NOTIFY isSectionChanged FINAL)
 
     /**
      * This property can only be used
      * in conjunction with a Kirigami.FormData.label,
-     * often in a layout that is a child of a FormLayout.
+     * often in a layout that is a child of a org::kde::kirigami::FormLayout.
      *
      * It then turns the item specified into a "buddy"
      * of the label, making it work as if it were
@@ -126,9 +102,13 @@ class FormLayoutAttached : public QObject
      * A buddy item is useful for instance when the label has a keyboard accelerator,
      * which when triggered provides active keyboard focus to the buddy item.
      *
+     * By default buddy is the item that Kirigami.FormData is attached to.
+     * Custom buddy can only be a direct child of that item; nested components
+     * are not supported at the moment.
+     *
      * @code
      * Kirigami.FormLayout {
-     *     ColumnLayout {
+     *     Layouts.ColumnLayout {
      *         // If the accelerator is in the letter S,
      *         // pressing Alt+S gives focus to the slider.
      *         Kirigami.FormData.label: "Slider label:"
@@ -144,7 +124,7 @@ class FormLayoutAttached : public QObject
      * }
      * @endcode
      */
-    Q_PROPERTY(QQuickItem *buddyFor READ buddyFor WRITE setBuddyFor NOTIFY buddyForChanged)
+    Q_PROPERTY(QQuickItem *buddyFor READ buddyFor WRITE setBuddyFor NOTIFY buddyForChanged FINAL)
 
 public:
     explicit FormLayoutAttached(QObject *parent = nullptr);
@@ -156,17 +136,8 @@ public:
     void setIsSection(bool section);
     bool isSection() const;
 
-    void setCheckable(bool checkable);
-    bool checkable() const;
-
-    void setChecked(bool checked);
-    bool checked() const;
-
-    void setEnabled(bool enabled);
-    bool enabled() const;
-
     QQuickItem *buddyFor() const;
-    void setBuddyFor(QQuickItem *buddyfor);
+    void setBuddyFor(QQuickItem *aBuddyFor);
 
     int labelAlignment() const;
     void setLabelAlignment(int alignment);
@@ -177,21 +148,17 @@ public:
 Q_SIGNALS:
     void labelChanged();
     void isSectionChanged();
-    void checkableChanged();
-    void checkedChanged();
-    void enabledChanged();
     void buddyForChanged();
     void labelAlignmentChanged();
 
 private:
+    void resetBuddyFor();
+
     QString m_label;
     QString m_actualDecoratedLabel;
     QString m_decoratedLabel;
     QPointer<QQuickItem> m_buddyFor;
     bool m_isSection = false;
-    bool m_checkable = false;
-    bool m_checked = false;
-    bool m_enabled = true;
     int m_labelAlignment = 0;
 };
 
