@@ -6,27 +6,29 @@
  */
 
 #include "icon.h"
-#include "libkirigami/platformtheme.h"
 #include "scenegraph/managedtexturenode.h"
 
+#include "platform/platformtheme.h"
+#include "platform/units.h"
+
 #include "loggingcategory.h"
 #include <QBitmap>
 #include <QDebug>
 #include <QGuiApplication>
 #include <QIcon>
+#include <QNetworkReply>
 #include <QPainter>
+#include <QPropertyAnimation>
 #include <QQuickImageProvider>
 #include <QQuickWindow>
 #include <QSGSimpleTextureNode>
 #include <QSGTexture>
 #include <QScreen>
-#include <QtQml>
 
 Q_GLOBAL_STATIC(ImageTexturesCache, s_iconImageCache)
 
 Icon::Icon(QQuickItem *parent)
     : QQuickItem(parent)
-    , m_changed(false)
     , m_active(false)
     , m_selected(false)
     , m_isMask(false)
@@ -34,37 +36,54 @@ Icon::Icon(QQuickItem *parent)
     setFlag(ItemHasContents, true);
     // Using 32 because Icon used to redefine implicitWidth and implicitHeight and hardcode them to 32
     setImplicitSize(32, 32);
-    // FIXME: not necessary anymore
-    connect(qApp, &QGuiApplication::paletteChanged, this, &QQuickItem::polish);
-    connect(this, &QQuickItem::enabledChanged, this, &QQuickItem::polish);
+
     connect(this, &QQuickItem::smoothChanged, this, &QQuickItem::polish);
+    connect(this, &QQuickItem::enabledChanged, this, [this]() {
+        m_allowNextAnimation = true;
+        polish();
+    });
 }
 
 Icon::~Icon()
 {
 }
 
+void Icon::componentComplete()
+{
+    QQuickItem::componentComplete();
+
+    QQmlEngine *engine = qmlEngine(this);
+    Q_ASSERT(engine);
+    m_units = engine->singletonInstance<Kirigami::Platform::Units *>("org.kde.kirigami.platform", "Units");
+    Q_ASSERT(m_units);
+    m_animation = new QPropertyAnimation(this);
+    connect(m_animation, &QPropertyAnimation::valueChanged, this, &Icon::valueChanged);
+    connect(m_animation, &QPropertyAnimation::finished, this, [this]() {
+        m_oldIcon = QImage();
+        m_textureChanged = true;
+        update();
+    });
+    m_animation->setTargetObject(this);
+    m_animation->setEasingCurve(QEasingCurve::InOutCubic);
+    m_animation->setDuration(m_units->longDuration());
+    connect(m_units, &Kirigami::Platform::Units::longDurationChanged, m_animation, [this]() {
+        m_animation->setDuration(m_units->longDuration());
+    });
+    updatePaintedGeometry();
+}
+
 void Icon::setSource(const QVariant &icon)
 {
     if (m_source == icon) {
         return;
     }
     m_source = icon;
-    m_monochromeHeuristics.clear();
 
     if (!m_theme) {
-        m_theme = static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true));
+        m_theme = static_cast<Kirigami::Platform::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::Platform::PlatformTheme>(this, true));
         Q_ASSERT(m_theme);
 
-        connect(m_theme, &Kirigami::PlatformTheme::colorsChanged, this, &QQuickItem::polish);
-    }
-
-    if (icon.type() == QVariant::String) {
-        const QString iconSource = icon.toString();
-        m_isMaskHeuristic = (iconSource.endsWith(QLatin1String("-symbolic")) //
-                             || iconSource.endsWith(QLatin1String("-symbolic-rtl")) //
-                             || iconSource.endsWith(QLatin1String("-symbolic-ltr")));
-        Q_EMIT isMaskChanged();
+        connect(m_theme, &Kirigami::Platform::PlatformTheme::colorsChanged, this, &QQuickItem::polish);
     }
 
     if (m_networkReply) {
@@ -90,6 +109,9 @@ void Icon::setActive(const bool active)
         return;
     }
     m_active = active;
+    if (isComponentComplete()) {
+        m_allowNextAnimation = true;
+    }
     polish();
     Q_EMIT activeChanged();
 }
@@ -132,14 +154,13 @@ void Icon::setIsMask(bool mask)
     }
 
     m_isMask = mask;
-    m_isMaskHeuristic = mask;
     polish();
     Q_EMIT isMaskChanged();
 }
 
 bool Icon::isMask() const
 {
-    return m_isMask || m_isMaskHeuristic;
+    return m_isMask;
 }
 
 void Icon::setColor(const QColor &color)
@@ -158,6 +179,30 @@ QColor Icon::color() const
     return m_color;
 }
 
+QSGNode *Icon::createSubtree(qreal initialOpacity)
+{
+    auto opacityNode = new QSGOpacityNode{};
+    opacityNode->setFlag(QSGNode::OwnedByParent, true);
+    opacityNode->setOpacity(initialOpacity);
+
+    auto *mNode = new ManagedTextureNode;
+
+    mNode->setTexture(s_iconImageCache->loadTexture(window(), m_icon, QQuickWindow::TextureCanUseAtlas));
+
+    opacityNode->appendChildNode(mNode);
+
+    return opacityNode;
+}
+
+void Icon::updateSubtree(QSGNode *node, qreal opacity)
+{
+    auto opacityNode = static_cast<QSGOpacityNode *>(node);
+    opacityNode->setOpacity(opacity);
+
+    auto textureNode = static_cast<ManagedTextureNode *>(opacityNode->firstChild());
+    textureNode->setFiltering(smooth() ? QSGTexture::Linear : QSGTexture::Nearest);
+}
+
 QSGNode *Icon::updatePaintNode(QSGNode *node, QQuickItem::UpdatePaintNodeData * /*data*/)
 {
     if (m_source.isNull() || qFuzzyIsNull(width()) || qFuzzyIsNull(height())) {
@@ -165,46 +210,77 @@ QSGNode *Icon::updatePaintNode(QSGNode *node, QQuickItem::UpdatePaintNodeData *
         return Q_NULLPTR;
     }
 
-    if (m_changed || node == nullptr) {
-        const QSize itemSize(width(), height());
-        QRect nodeRect(QPoint(0, 0), itemSize);
+    if (!node) {
+        node = new QSGNode{};
+    }
 
-        ManagedTextureNode *mNode = dynamic_cast<ManagedTextureNode *>(node);
-        if (!mNode) {
-            delete node;
-            mNode = new ManagedTextureNode;
+    if (m_animation && m_animation->state() == QAbstractAnimation::Running) {
+        if (node->childCount() < 2) {
+            node->appendChildNode(createSubtree(0.0));
+            m_textureChanged = true;
         }
-        if (itemSize.width() != 0 && itemSize.height() != 0) {
-            mNode->setTexture(s_iconImageCache->loadTexture(window(), m_icon, QQuickWindow::TextureCanUseAtlas));
-            if (m_icon.size() != itemSize) {
+
+        // Rather than doing a perfect crossfade, first fade in the new texture
+        // then fade out the old texture. This is done to avoid the underlying
+        // color bleeding through when both textures are at ~0.5 opacity, which
+        // causes flickering if the two textures are very similar.
+        updateSubtree(node->firstChild(), 2.0 - m_animValue * 2.0);
+        updateSubtree(node->lastChild(), m_animValue * 2.0);
+    } else {
+        if (node->childCount() == 0) {
+            node->appendChildNode(createSubtree(1.0));
+            m_textureChanged = true;
+        }
+
+        if (node->childCount() > 1) {
+            auto toRemove = node->firstChild();
+            node->removeChildNode(toRemove);
+            delete toRemove;
+        }
+
+        updateSubtree(node->firstChild(), 1.0);
+    }
+
+    if (m_textureChanged) {
+        auto mNode = static_cast<ManagedTextureNode *>(node->lastChild()->firstChild());
+        mNode->setTexture(s_iconImageCache->loadTexture(window(), m_icon, QQuickWindow::TextureCanUseAtlas));
+        m_textureChanged = false;
+        m_sizeChanged = true;
+    }
+
+    if (m_sizeChanged) {
+        const QSizeF iconPixSize(m_icon.width() / m_devicePixelRatio, m_icon.height() / m_devicePixelRatio);
+        const QSizeF itemPixSize = QSizeF((size() * m_devicePixelRatio).toSize()) / m_devicePixelRatio;
+        QRectF nodeRect(QPoint(0, 0), itemPixSize);
+
+        if (itemPixSize.width() != 0 && itemPixSize.height() != 0) {
+            if (iconPixSize != itemPixSize) {
                 // At this point, the image will already be scaled, but we need to output it in
                 // the correct aspect ratio, painted centered in the viewport. So:
-                QRect destination(QPoint(0, 0), m_icon.size().scaled(itemSize, Qt::KeepAspectRatio));
+                QRectF destination(QPointF(0, 0), QSizeF(m_icon.size()).scaled(m_paintedSize, Qt::KeepAspectRatio));
                 destination.moveCenter(nodeRect.center());
+                destination.moveTopLeft(QPointF(destination.topLeft().toPoint() * m_devicePixelRatio) / m_devicePixelRatio);
                 nodeRect = destination;
             }
         }
-        mNode->setRect(nodeRect);
-        node = mNode;
-        if (smooth()) {
-            mNode->setFiltering(QSGTexture::Linear);
+
+        for (int i = 0; i < node->childCount(); ++i) {
+            auto mNode = static_cast<ManagedTextureNode *>(node->childAtIndex(i)->firstChild());
+            mNode->setRect(nodeRect);
         }
-        m_changed = false;
+
+        m_sizeChanged = false;
     }
 
     return node;
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-void Icon::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
-{
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
-#else
 void Icon::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     QQuickItem::geometryChange(newGeometry, oldGeometry);
-#endif
     if (newGeometry.size() != oldGeometry.size()) {
+        m_sizeChanged = true;
+        updatePaintedGeometry();
         polish();
     }
 }
@@ -252,8 +328,7 @@ void Icon::handleFinished(QNetworkReply *reply)
         qCWarning(KirigamiLog) << "received broken image" << reply->url();
 
         // broken image from data, inform the user of this with some useful broken-image thing...
-        const QIcon icon = QIcon::fromTheme(m_fallback);
-        m_loadedImage = icon.pixmap(window(), icon.actualSize(size().toSize()), iconMode(), QIcon::On).toImage();
+        m_loadedImage = iconPixmap(QIcon::fromTheme(m_fallback));
     }
 
     polish();
@@ -263,6 +338,10 @@ void Icon::updatePolish()
 {
     QQuickItem::updatePolish();
 
+    if (window()) {
+        m_devicePixelRatio = window()->effectiveDevicePixelRatio();
+    }
+
     if (m_source.isNull()) {
         setStatus(Ready);
         updatePaintedGeometry();
@@ -272,33 +351,34 @@ void Icon::updatePolish()
 
     const QSize itemSize(width(), height());
     if (itemSize.width() != 0 && itemSize.height() != 0) {
-        const auto multiplier = QCoreApplication::instance()->testAttribute(Qt::AA_UseHighDpiPixmaps)
-            ? 1
-            : (window() ? window()->effectiveDevicePixelRatio() : qGuiApp->devicePixelRatio());
-        const QSize size = itemSize * multiplier;
+        const QSize size = itemSize;
 
-        switch (m_source.type()) {
-        case QVariant::Pixmap:
+        if (m_animation) {
+            m_animation->stop();
+            m_oldIcon = m_icon;
+        }
+
+        switch (m_source.userType()) {
+        case QMetaType::QPixmap:
             m_icon = m_source.value<QPixmap>().toImage();
             break;
-        case QVariant::Image:
+        case QMetaType::QImage:
             m_icon = m_source.value<QImage>();
             break;
-        case QVariant::Bitmap:
+        case QMetaType::QBitmap:
             m_icon = m_source.value<QBitmap>().toImage();
             break;
-        case QVariant::Icon: {
-            const QIcon icon = m_source.value<QIcon>();
-            m_icon = icon.pixmap(window(), icon.actualSize(itemSize), iconMode(), QIcon::On).toImage();
+        case QMetaType::QIcon: {
+            m_icon = iconPixmap(m_source.value<QIcon>());
             break;
         }
-        case QVariant::Url:
-        case QVariant::String:
+        case QMetaType::QUrl:
+        case QMetaType::QString:
             m_icon = findIcon(size);
             break;
-        case QVariant::Brush:
+        case QMetaType::QBrush:
             // todo: fill here too?
-        case QVariant::Color:
+        case QMetaType::QColor:
             m_icon = QImage(size, QImage::Format_Alpha8);
             m_icon.fill(m_source.value<QColor>());
             break;
@@ -317,14 +397,31 @@ void Icon::updatePolish()
             : m_color;
 
         // TODO: initialize m_isMask with icon.isMask()
-        if (tintColor.alpha() > 0 && (isMask() || guessMonochrome(m_icon))) {
+        if (tintColor.alpha() > 0 && isMask()) {
             QPainter p(&m_icon);
             p.setCompositionMode(QPainter::CompositionMode_SourceIn);
             p.fillRect(m_icon.rect(), tintColor);
             p.end();
         }
     }
-    m_changed = true;
+
+    // don't animate initial setting
+    bool animated = (m_animated || m_allowNextAnimation) && !m_oldIcon.isNull() && !m_sizeChanged && !m_blockNextAnimation;
+
+    if (animated && m_animation) {
+        m_animValue = 0.0;
+        m_animation->setStartValue((qreal)0);
+        m_animation->setEndValue((qreal)1);
+        m_animation->start();
+        m_allowNextAnimation = false;
+    } else {
+        if (m_animation) {
+            m_animation->stop();
+        }
+        m_animValue = 1.0;
+        m_blockNextAnimation = false;
+    }
+    m_textureChanged = true;
     updatePaintedGeometry();
     update();
 }
@@ -335,9 +432,6 @@ QImage Icon::findIcon(const QSize &size)
     QString iconSource = m_source.toString();
 
     if (iconSource.startsWith(QLatin1String("image://"))) {
-        const auto multiplier = QCoreApplication::instance()->testAttribute(Qt::AA_UseHighDpiPixmaps)
-            ? (window() ? window()->effectiveDevicePixelRatio() : qGuiApp->devicePixelRatio())
-            : 1;
         QUrl iconUrl(iconSource);
         QString iconProviderId = iconUrl.host();
         // QUrl path has the  "/" prefix while iconId does not
@@ -350,13 +444,13 @@ QImage Icon::findIcon(const QSize &size)
         }
         switch (imageProvider->imageType()) {
         case QQmlImageProviderBase::Image:
-            img = imageProvider->requestImage(iconId, &actualSize, size * multiplier);
+            img = imageProvider->requestImage(iconId, &actualSize, size);
             if (!img.isNull()) {
                 setStatus(Ready);
             }
             break;
         case QQmlImageProviderBase::Pixmap:
-            img = imageProvider->requestPixmap(iconId, &actualSize, size * multiplier).toImage();
+            img = imageProvider->requestPixmap(iconId, &actualSize, size).toImage();
             if (!img.isNull()) {
                 setStatus(Ready);
             }
@@ -367,7 +461,7 @@ QImage Icon::findIcon(const QSize &size)
                 return m_loadedImage.scaled(size, Qt::KeepAspectRatio, smooth() ? Qt::SmoothTransformation : Qt::FastTransformation);
             }
             QQuickAsyncImageProvider *provider = dynamic_cast<QQuickAsyncImageProvider *>(imageProvider);
-            auto response = provider->requestImageResponse(iconId, size * multiplier);
+            auto response = provider->requestImageResponse(iconId, size);
             connect(response, &QQuickImageResponse::finished, this, [iconId, response, this]() {
                 if (response->errorString().isEmpty()) {
                     QQuickTextureFactory *textureFactory = response->textureFactory();
@@ -377,8 +471,7 @@ QImage Icon::findIcon(const QSize &size)
                     }
                     if (m_loadedImage.isNull()) {
                         // broken image from data, inform the user of this with some useful broken-image thing...
-                        const QIcon icon = QIcon::fromTheme(m_fallback);
-                        m_loadedImage = icon.pixmap(window(), icon.actualSize(QSize(width(), height())), iconMode(), QIcon::On).toImage();
+                        m_loadedImage = iconPixmap(QIcon::fromTheme(m_fallback));
                         setStatus(Error);
                     } else {
                         setStatus(Ready);
@@ -388,19 +481,17 @@ QImage Icon::findIcon(const QSize &size)
                 response->deleteLater();
             });
             // Temporary icon while we wait for the real image to load...
-            const QIcon icon = QIcon::fromTheme(m_placeholder);
-            img = icon.pixmap(window(), icon.actualSize(size), iconMode(), QIcon::On).toImage();
+            img = iconPixmap(QIcon::fromTheme(m_placeholder));
             break;
         }
         case QQmlImageProviderBase::Texture: {
-            QQuickTextureFactory *textureFactory = imageProvider->requestTexture(iconId, &actualSize, size * multiplier);
+            QQuickTextureFactory *textureFactory = imageProvider->requestTexture(iconId, &actualSize, size);
             if (textureFactory) {
                 img = textureFactory->image();
             }
             if (img.isNull()) {
                 // broken image from data, or the texture factory wasn't healthy, inform the user of this with some useful broken-image thing...
-                const QIcon icon = QIcon::fromTheme(m_fallback);
-                img = icon.pixmap(window(), icon.actualSize(QSize(width(), height())), iconMode(), QIcon::On).toImage();
+                img = iconPixmap(QIcon::fromTheme(m_fallback));
                 setStatus(Error);
             } else {
                 setStatus(Ready);
@@ -429,8 +520,7 @@ QImage Icon::findIcon(const QSize &size)
             });
         }
         // Temporary icon while we wait for the real image to load...
-        const QIcon icon = QIcon::fromTheme(m_placeholder);
-        img = icon.pixmap(window(), icon.actualSize(size), iconMode(), QIcon::On).toImage();
+        img = iconPixmap(QIcon::fromTheme(m_placeholder));
     } else {
         if (iconSource.startsWith(QLatin1String("qrc:/"))) {
             iconSource = iconSource.mid(3);
@@ -443,31 +533,19 @@ QImage Icon::findIcon(const QSize &size)
         if (isPath) {
             icon = QIcon(iconSource);
         } else {
-            if (icon.isNull()) {
-                icon = m_theme->iconFromTheme(iconSource, m_color);
-            }
+            const QColor tintColor =
+                !m_color.isValid() || m_color == Qt::transparent ? (m_selected ? m_theme->highlightedTextColor() : m_theme->textColor()) : m_color;
+            icon = m_theme->iconFromTheme(iconSource, tintColor);
         }
         if (!icon.isNull()) {
-            img = icon.pixmap(window(), icon.actualSize(window(), size), iconMode(), QIcon::On).toImage();
-
+            img = iconPixmap(icon);
             setStatus(Ready);
-            /*const QColor tintColor = !m_color.isValid() || m_color == Qt::transparent ? (m_selected ? m_theme->highlightedTextColor() : m_theme->textColor())
-            : m_color;
-
-            if (m_isMask || icon.isMask() || iconSource.endsWith(QLatin1String("-symbolic")) || iconSource.endsWith(QLatin1String("-symbolic-rtl")) ||
-            iconSource.endsWith(QLatin1String("-symbolic-ltr")) || guessMonochrome(img)) { //
-                QPainter p(&img);
-                p.setCompositionMode(QPainter::CompositionMode_SourceIn);
-                p.fillRect(img.rect(), tintColor);
-                p.end();
-            }*/
         }
     }
 
     if (!iconSource.isEmpty() && img.isNull()) {
         setStatus(Error);
-        const QIcon icon = QIcon::fromTheme(m_fallback);
-        img = icon.pixmap(window(), icon.actualSize(size), iconMode(), QIcon::On).toImage();
+        img = iconPixmap(QIcon::fromTheme(m_fallback));
     }
     return img;
 }
@@ -484,66 +562,6 @@ QIcon::Mode Icon::iconMode() const
     return QIcon::Normal;
 }
 
-bool Icon::guessMonochrome(const QImage &img)
-{
-    // don't try for too big images
-    if (img.width() >= 256 || m_theme->supportsIconColoring()) {
-        return false;
-    }
-    // round size to a standard size. hardcode as we can't use KIconLoader
-    int stdSize;
-    if (img.width() <= 16) {
-        stdSize = 16;
-    } else if (img.width() <= 22) {
-        stdSize = 22;
-    } else if (img.width() <= 24) {
-        stdSize = 24;
-    } else if (img.width() <= 32) {
-        stdSize = 32;
-    } else if (img.width() <= 48) {
-        stdSize = 48;
-    } else if (img.width() <= 64) {
-        stdSize = 64;
-    } else {
-        stdSize = 128;
-    }
-
-    auto findIt = m_monochromeHeuristics.constFind(stdSize);
-    if (findIt != m_monochromeHeuristics.constEnd()) {
-        return findIt.value();
-    }
-
-    QHash<int, int> dist;
-    int transparentPixels = 0;
-    int saturatedPixels = 0;
-    for (int x = 0; x < img.width(); x++) {
-        for (int y = 0; y < img.height(); y++) {
-            QColor color = QColor::fromRgba(qUnpremultiply(img.pixel(x, y)));
-            if (color.alpha() < 100) {
-                ++transparentPixels;
-                continue;
-            } else if (color.saturation() > 84) {
-                ++saturatedPixels;
-            }
-            dist[qGray(color.rgb())]++;
-        }
-    }
-
-    QMultiMap<int, int> reverseDist;
-    auto it = dist.constBegin();
-    qreal entropy = 0;
-    while (it != dist.constEnd()) {
-        reverseDist.insert(it.value(), it.key());
-        qreal probability = qreal(it.value()) / qreal(img.size().width() * img.size().height() - transparentPixels);
-        entropy -= probability * log(probability) / log(255);
-        ++it;
-    }
-
-    // Arbitrarily low values of entropy and colored pixels
-    m_monochromeHeuristics[stdSize] = saturatedPixels <= (img.size().width() * img.size().height() - transparentPixels) * 0.3 && entropy <= 0.3;
-    return m_monochromeHeuristics[stdSize];
-}
-
 QString Icon::fallback() const
 {
     return m_fallback;
@@ -587,46 +605,148 @@ Icon::Status Icon::status() const
 
 qreal Icon::paintedWidth() const
 {
-    return m_paintedWidth;
+    return std::round(m_paintedSize.width());
 }
 
 qreal Icon::paintedHeight() const
 {
-    return m_paintedHeight;
+    return std::round(m_paintedSize.height());
+}
+
+QSize Icon::iconSizeHint() const
+{
+    // size hint is always square, non square case in updatePaintGeometry
+    if (m_roundToIconSize && m_units) {
+        return QSize(m_units->iconSizes()->roundedIconSize(std::min(width(), height())), m_units->iconSizes()->roundedIconSize(std::min(width(), height())));
+    } else {
+        return QSize(std::min(width(), height()), std::min(width(), height()));
+    }
+}
+
+QImage Icon::iconPixmap(const QIcon &icon) const
+{
+    const QSize actualSize = icon.actualSize(iconSizeHint());
+    return icon.pixmap(actualSize, m_devicePixelRatio, iconMode(), QIcon::On).toImage();
 }
 
 void Icon::updatePaintedGeometry()
 {
-    qreal newWidth = 0.0;
-    qreal newHeight = 0.0;
+    QSizeF newSize;
     if (!m_icon.width() || !m_icon.height()) {
-        newWidth = newHeight = 0.0;
+        newSize = {0, 0};
     } else {
-        const qreal w = widthValid() ? width() : m_icon.size().width();
-        const qreal widthScale = w / m_icon.size().width();
-        const qreal h = heightValid() ? height() : m_icon.size().height();
-        const qreal heightScale = h / m_icon.size().height();
-        if (widthScale <= heightScale) {
-            newWidth = w;
-            newHeight = widthScale * m_icon.size().height();
-        } else if (heightScale < widthScale) {
-            newWidth = heightScale * m_icon.size().width();
-            newHeight = h;
+        qreal roundedWidth = m_units ? m_units->iconSizes()->roundedIconSize(std::min(width(), height())) : 32;
+        roundedWidth = std::round(roundedWidth * m_devicePixelRatio) / m_devicePixelRatio;
+
+        if (QSizeF roundedSize(roundedWidth, roundedWidth); size() == roundedSize) {
+            m_paintedSize = roundedSize;
+            m_textureChanged = true;
+            update();
+            Q_EMIT paintedAreaChanged();
+            return;
+        }
+        if (m_roundToIconSize && m_units) {
+            if (m_icon.width() > m_icon.height()) {
+                newSize = QSizeF(roundedWidth, m_icon.height() * (roundedWidth / static_cast<qreal>(m_icon.width())));
+            } else {
+                newSize = QSizeF(roundedWidth, roundedWidth);
+            }
+        } else {
+            const QSizeF iconPixSize(m_icon.width() / m_devicePixelRatio, m_icon.width() / m_devicePixelRatio);
+
+            const qreal w = widthValid() ? width() : iconPixSize.width();
+            const qreal widthScale = w / iconPixSize.width();
+            const qreal h = heightValid() ? height() : iconPixSize.height();
+            const qreal heightScale = h / iconPixSize.height();
+
+            if (widthScale <= heightScale) {
+                newSize = QSizeF(w, widthScale * iconPixSize.height());
+            } else if (heightScale < widthScale) {
+                newSize = QSizeF(heightScale * iconPixSize.width(), h);
+            }
         }
     }
-    if (newWidth != m_paintedWidth || newHeight != m_paintedHeight) {
-        m_paintedWidth = newWidth;
-        m_paintedHeight = newHeight;
+    if (newSize != m_paintedSize) {
+        m_paintedSize = newSize;
+        m_textureChanged = true;
+        update();
         Q_EMIT paintedAreaChanged();
     }
 }
 
+bool Icon::isAnimated() const
+{
+    return m_animated;
+}
+
+void Icon::setAnimated(bool animated)
+{
+    if (m_animated == animated) {
+        return;
+    }
+
+    m_animated = animated;
+    Q_EMIT animatedChanged();
+}
+
+bool Icon::roundToIconSize() const
+{
+    return m_roundToIconSize;
+}
+
+void Icon::setRoundToIconSize(bool roundToIconSize)
+{
+    if (m_roundToIconSize == roundToIconSize) {
+        return;
+    }
+
+    const QSizeF oldPaintedSize = m_paintedSize;
+
+    m_roundToIconSize = roundToIconSize;
+    Q_EMIT roundToIconSizeChanged();
+
+    updatePaintedGeometry();
+    if (oldPaintedSize != m_paintedSize) {
+        Q_EMIT paintedAreaChanged();
+        m_textureChanged = true;
+        update();
+    }
+}
+
 void Icon::itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &value)
 {
     if (change == QQuickItem::ItemDevicePixelRatioHasChanged) {
+        m_blockNextAnimation = true;
+        if (window()) {
+            m_devicePixelRatio = window()->effectiveDevicePixelRatio();
+        }
         polish();
+    } else if (change == QQuickItem::ItemSceneChange) {
+        if (m_window) {
+            disconnect(m_window.data(), &QWindow::visibleChanged, this, &Icon::windowVisibleChanged);
+        }
+        m_window = value.window;
+        if (m_window) {
+            connect(m_window.data(), &QWindow::visibleChanged, this, &Icon::windowVisibleChanged);
+            m_devicePixelRatio = m_window->effectiveDevicePixelRatio();
+        }
+    } else if (change == ItemVisibleHasChanged && value.boolValue) {
+        m_blockNextAnimation = true;
     }
     QQuickItem::itemChange(change, value);
 }
 
+void Icon::valueChanged(const QVariant &value)
+{
+    m_animValue = value.toReal();
+    update();
+}
+
+void Icon::windowVisibleChanged(bool visible)
+{
+    if (visible) {
+        m_blockNextAnimation = true;
+    }
+}
+
 #include "moc_icon.cpp"
