@@ -17,7 +17,6 @@
  */
 
 #include "imagecolors.h"
-#include "platformtheme.h"
 
 #include <QDebug>
 #include <QFutureWatcher>
@@ -34,6 +33,8 @@
 #include <omp.h>
 #endif
 
+#include "platform/platformtheme.h"
+
 #define return_fallback(value)                                                                                                                                 \
     if (m_imageData.m_samples.size() == 0) {                                                                                                                   \
         return value;                                                                                                                                          \
@@ -41,7 +42,9 @@
 
 #define return_fallback_finally(value, finally)                                                                                                                \
     if (m_imageData.m_samples.size() == 0) {                                                                                                                   \
-        return value.isValid() ? value : static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true))->finally();  \
+        return value.isValid()                                                                                                                                 \
+            ? value                                                                                                                                            \
+            : static_cast<Kirigami::Platform::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::Platform::PlatformTheme>(this, true))->finally();         \
     }
 
 ImageColors::ImageColors(QObject *parent)
@@ -224,7 +227,7 @@ void ImageColors::update()
 inline int squareDistance(QRgb color1, QRgb color2)
 {
     // https://en.wikipedia.org/wiki/Color_difference
-    // Using RGB distance for performance, as CIEDE2000 istoo complicated
+    // Using RGB distance for performance, as CIEDE2000 is too complicated
     if (qRed(color1) - qRed(color2) < 128) {
         return 2 * pow(qRed(color1) - qRed(color2), 2) //
             + 4 * pow(qGreen(color1) - qGreen(color2), 2) //
@@ -251,6 +254,64 @@ void ImageColors::positionColor(QRgb rgb, QList<ImageData::colorStat> &clusters)
     clusters << stat;
 }
 
+void ImageColors::positionColorMP(const decltype(ImageData::m_samples) &samples, decltype(ImageData::m_clusters) &clusters, int numCore)
+{
+#if HAVE_OpenMP
+    if (samples.size() < 65536 /* 256^2 */ || numCore < 2) {
+#else
+    if (true) {
+#endif
+        // Fall back to single thread
+        for (auto color : samples) {
+            positionColor(color, clusters);
+        }
+        return;
+    }
+#if HAVE_OpenMP
+    // Split the whole samples into multiple parts
+    const int numSamplesPerThread = samples.size() / numCore;
+    std::vector<decltype(ImageData::m_clusters)> tempClusters(numCore, decltype(ImageData::m_clusters){});
+#pragma omp parallel for
+    for (int i = 0; i < numCore; ++i) {
+        decltype(ImageData::m_samples) samplePart;
+        const auto beginIt = std::next(samples.begin(), numSamplesPerThread * i);
+        const auto endIt = i < numCore - 1 ? std::next(samples.begin(), numSamplesPerThread * (i + 1)) : samples.end();
+
+        for (auto it = beginIt; it != endIt; it = std::next(it)) {
+            positionColor(*it, tempClusters[omp_get_thread_num()]);
+        }
+    } // END omp parallel for
+
+    // Restore clusters
+    // Don't use std::as_const as memory will grow significantly
+    for (const auto &clusterPart : tempClusters) {
+        clusters << clusterPart;
+    }
+    for (int i = 0; i < clusters.size() - 1; ++i) {
+        auto &clusterA = clusters[i];
+        if (clusterA.colors.empty()) {
+            continue; // Already merged
+        }
+        for (int j = i + 1; j < clusters.size(); ++j) {
+            auto &clusterB = clusters[j];
+            if (clusterB.colors.empty()) {
+                continue; // Already merged
+            }
+            if (squareDistance(clusterA.centroid, clusterB.centroid) < s_minimumSquareDistance) {
+                // Merge colors in clusterB into clusterA
+                clusterA.colors.append(clusterB.colors);
+                clusterB.colors.clear();
+            }
+        }
+    }
+
+    auto removeIt = std::remove_if(clusters.begin(), clusters.end(), [](const ImageData::colorStat &stat) {
+        return stat.colors.empty();
+    });
+    clusters.erase(removeIt, clusters.end());
+#endif
+}
+
 ImageData ImageColors::generatePalette(const QImage &sourceImage) const
 {
     ImageData imageData;
@@ -265,7 +326,8 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage) const
 #if HAVE_OpenMP
     static const int numCore = std::min(8, omp_get_num_procs());
     omp_set_num_threads(numCore);
-    std::vector<decltype(imageData.m_samples)> tempSamples(numCore, decltype(imageData.m_samples){});
+#else
+    constexpr int numCore = 1;
 #endif
     int r = 0;
     int g = 0;
@@ -287,27 +349,16 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage) const
             r += qRed(rgb);
             g += qGreen(rgb);
             b += qBlue(rgb);
-#if HAVE_OpenMP
-            tempSamples[omp_get_thread_num()] << rgb;
-#else
+#pragma omp critical
             imageData.m_samples << rgb;
-#endif
         }
     } // END omp parallel for
 
-#if HAVE_OpenMP
-    for (auto &s : tempSamples) {
-        imageData.m_samples << std::move(s);
-    }
-#endif
-
     if (imageData.m_samples.isEmpty()) {
         return imageData;
     }
 
-    for (QRgb rgb : std::as_const(imageData.m_samples)) {
-        positionColor(rgb, imageData.m_clusters);
-    }
+    positionColorMP(imageData.m_samples, imageData.m_clusters, numCore);
 
     imageData.m_average = QColor(r / c, g / c, b / c, 255);
 
@@ -334,9 +385,7 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage) const
             stat.colors = QList<QRgb>({stat.centroid});
         } // END omp parallel for
 
-        for (auto color : std::as_const(imageData.m_samples)) {
-            positionColor(color, imageData.m_clusters);
-        }
+        positionColorMP(imageData.m_samples, imageData.m_clusters, numCore);
     }
 
     std::sort(imageData.m_clusters.begin(), imageData.m_clusters.end(), [this](const ImageData::colorStat &a, const ImageData::colorStat &b) {
@@ -375,7 +424,9 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage) const
 
     bool first = true;
 
-    for (const auto &stat : std::as_const(imageData.m_clusters)) {
+#pragma omp parallel for ordered
+    for (int i = 0; i < imageData.m_clusters.size(); ++i) {
+        const auto &stat = imageData.m_clusters[i];
         QVariantMap entry;
         const QColor color(stat.centroid);
         entry[QStringLiteral("color")] = color;
@@ -413,24 +464,26 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage) const
         }
 
         entry[QStringLiteral("contrastColor")] = contrast;
+#pragma omp ordered
+        { // BEGIN omp ordered
+            if (first) {
+                imageData.m_dominantContrast = contrast;
+                imageData.m_dominant = color;
+            }
+            first = false;
 
-        if (first) {
-            imageData.m_dominantContrast = contrast;
-            imageData.m_dominant = color;
-        }
-        first = false;
+            if (!imageData.m_highlight.isValid() || ColorUtils::chroma(color) > ColorUtils::chroma(imageData.m_highlight)) {
+                imageData.m_highlight = color;
+            }
 
-        if (!imageData.m_highlight.isValid() || ColorUtils::chroma(color) > ColorUtils::chroma(imageData.m_highlight)) {
-            imageData.m_highlight = color;
-        }
-
-        if (qGray(color.rgb()) > qGray(imageData.m_closestToWhite.rgb())) {
-            imageData.m_closestToWhite = color;
-        }
-        if (qGray(color.rgb()) < qGray(imageData.m_closestToBlack.rgb())) {
-            imageData.m_closestToBlack = color;
-        }
-        imageData.m_palette << entry;
+            if (qGray(color.rgb()) > qGray(imageData.m_closestToWhite.rgb())) {
+                imageData.m_closestToWhite = color;
+            }
+            if (qGray(color.rgb()) < qGray(imageData.m_closestToBlack.rgb())) {
+                imageData.m_closestToBlack = color;
+            }
+            imageData.m_palette << entry;
+        } // END omp ordered
     }
 
     postProcess(imageData);
@@ -447,7 +500,8 @@ void ImageColors::postProcess(ImageData &imageData) const
 {
     constexpr short unsigned WCAG_NON_TEXT_CONTRAST_RATIO = 3;
     constexpr qreal WCAG_TEXT_CONTRAST_RATIO = 4.5;
-    const QColor backgroundColor = static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true))->backgroundColor();
+    const QColor backgroundColor =
+        static_cast<Kirigami::Platform::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::Platform::PlatformTheme>(this, true))->backgroundColor();
     const qreal backgroundLum = ColorUtils::luminance(backgroundColor);
     qreal lowerLum, upperLum;
     // 192 is from kcm_colors
@@ -458,7 +512,8 @@ void ImageColors::postProcess(ImageData &imageData) const
     } else {
         // For light themes, still prefer lighter colors
         // (lowerLum + 0.05) / (textLum + 0.05) >= 4.5
-        const QColor textColor = static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true))->textColor();
+        const QColor textColor =
+            static_cast<Kirigami::Platform::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::Platform::PlatformTheme>(this, true))->textColor();
         const qreal textLum = ColorUtils::luminance(textColor);
         lowerLum = WCAG_TEXT_CONTRAST_RATIO * (textLum + 0.05) - 0.05;
         upperLum = backgroundLum;
