@@ -7,26 +7,23 @@
  */
 
 #include "kirigamiplugin.h"
-#include "avatar.h"
 #include "colorutils.h"
 #include "columnview.h"
-#include "delegaterecycler.h"
 #include "enums.h"
 #include "formlayoutattached.h"
+#include "headerfooterlayout.h"
 #include "icon.h"
 #include "imagecolors.h"
-#include "inputmethod.h"
 #include "mnemonicattached.h"
+#include "overlayzstackingattached.h"
+#include "padding.h"
 #include "pagepool.h"
-#include "pagerouter.h"
 #include "scenepositionattached.h"
-#include "settings.h"
 #include "shadowedrectangle.h"
 #include "shadowedtexture.h"
 #include "sizegroup.h"
-#include "spellcheckinghint.h"
+#include "spellcheckattached.h"
 #include "toolbarlayout.h"
-#include "units.h"
 #include "wheelhandler.h"
 
 #include <QClipboard>
@@ -38,12 +35,9 @@
 #include <QQuickItem>
 #include <QQuickStyle>
 
-#include "libkirigami/basictheme_p.h"
-#include "libkirigami/platformtheme.h"
-#include "libkirigami/styleselector_p.h"
+#include "platform/styleselector.h"
+
 #include "loggingcategory.h"
-#include "libkirigami/basictheme_p.h"
-#include "libkirigami/kirigamipluginfactory.h"
 
 static QString s_selectedStyle;
 
@@ -91,7 +85,7 @@ KirigamiPlugin::KirigamiPlugin(QObject *parent)
 
 QUrl KirigamiPlugin::componentUrl(const QString &fileName) const
 {
-    return Kirigami::StyleSelector::componentUrl(fileName);
+    return Kirigami::Platform::StyleSelector::componentUrl(fileName);
 }
 
 using SingletonCreationFunction = QObject *(*)(QQmlEngine *, QJSEngine *);
@@ -115,68 +109,31 @@ void KirigamiPlugin::registerTypes(const char *uri)
 
     Q_ASSERT(QLatin1String(uri) == QLatin1String("org.kde.kirigami"));
 
-    Kirigami::StyleSelector::setBaseUrl(baseUrl());
+    Kirigami::Platform::StyleSelector::setBaseUrl(baseUrl());
 
     if (QIcon::themeName().isEmpty() && !qEnvironmentVariableIsSet("XDG_CURRENT_DESKTOP")) {
-        QIcon::setThemeSearchPaths({Kirigami::StyleSelector::resolveFilePath(QStringLiteral(".")), QStringLiteral(":/icons")});
+#if defined(Q_OS_ANDROID)
+        QIcon::setThemeSearchPaths({QStringLiteral("assets:/qml/org/kde/kirigami"), QStringLiteral(":/icons")});
+#else
+        QIcon::setThemeSearchPaths({Kirigami::Platform::StyleSelector::resolveFilePath(QStringLiteral(".")), QStringLiteral(":/icons")});
+#endif
         QIcon::setThemeName(QStringLiteral("breeze-internal"));
     } else {
-        QIcon::setFallbackSearchPaths(QIcon::fallbackSearchPaths() << Kirigami::StyleSelector::resolveFilePath(QStringLiteral("icons")));
+        QIcon::setFallbackSearchPaths(QIcon::fallbackSearchPaths() << Kirigami::Platform::StyleSelector::resolveFilePath(QStringLiteral("icons")));
     }
 
-    qmlRegisterSingletonType<Settings>(uri, 2, 0, "Settings", [](QQmlEngine *e, QJSEngine *) -> QObject * {
-        Settings *settings = Settings::self();
-        // singleton managed internally, qml should never delete it
-        e->setObjectOwnership(settings, QQmlEngine::CppOwnership);
-        settings->setStyle(Kirigami::StyleSelector::style());
-        return settings;
-    });
-
-    qmlRegisterUncreatableType<ApplicationHeaderStyle>(uri,
-                                                       2,
-                                                       0,
-                                                       "ApplicationHeaderStyle",
-                                                       QStringLiteral("Cannot create objects of type ApplicationHeaderStyle"));
-
-    // old legacy retrocompatible Theme
-    qmlRegisterSingletonType<Kirigami::BasicThemeDefinition>(uri, 2, 0, "Theme", [](QQmlEngine *, QJSEngine *) {
-        qCWarning(KirigamiLog) << "The Theme singleton is deprecated (since 5.39). Import Kirigami 2.2 or higher and use the attached property instead.";
-        return new Kirigami::BasicThemeDefinition{};
-    });
-
-    qmlRegisterSingletonType<Kirigami::Units>(uri, 2, 0, "Units", [] (QQmlEngine *engine, QJSEngine *) {
-#ifndef KIRIGAMI_BUILD_TYPE_STATIC
-        auto plugin = Kirigami::KirigamiPluginFactory::findPlugin();
-        if (plugin) {
-            // Check if the plugin implements units
-            auto pluginV2 = qobject_cast<Kirigami::KirigamiPluginFactoryV2 *>(plugin);
-            if (pluginV2) {
-                auto units = pluginV2->createUnits(engine);
-                if (units) {
-                    return units;
-                } else {
-                    qWarning(KirigamiLog) << "The style returned a nullptr Units*, falling back to defaults";
-                }
-            } else {
-                qWarning(KirigamiLog) << "The style does not provide a C++ Units implementation."
-                                      << "QML Units implementations are no longer supported.";
-            }
-        } else {
-            qWarning(KirigamiLog) << "Failed to find a Kirigami platform plugin";
-        }
-#endif
-        // Fall back to the default units implementation
-        return new Kirigami::Units(engine);
-    });
+    qmlRegisterUncreatableMetaObject(ApplicationHeaderStyle::staticMetaObject,
+                                     uri,
+                                     2,
+                                     0,
+                                     "ApplicationHeaderStyle",
+                                     QStringLiteral("Cannot create objects of type ApplicationHeaderStyle"));
 
+    qmlRegisterRevision<QQuickItem, 6>(uri, 2, 0);
     qmlRegisterType(componentUrl(QStringLiteral("Action.qml")), uri, 2, 0, "Action");
     qmlRegisterType(componentUrl(QStringLiteral("AbstractApplicationHeader.qml")), uri, 2, 0, "AbstractApplicationHeader");
     qmlRegisterType(componentUrl(QStringLiteral("AbstractApplicationWindow.qml")), uri, 2, 0, "AbstractApplicationWindow");
-    qmlRegisterType(componentUrl(QStringLiteral("AbstractListItem.qml")), uri, 2, 0, "AbstractListItem");
-    qmlRegisterType(componentUrl(QStringLiteral("ApplicationHeader.qml")), uri, 2, 0, "ApplicationHeader");
-    qmlRegisterType(componentUrl(QStringLiteral("ToolBarApplicationHeader.qml")), uri, 2, 0, "ToolBarApplicationHeader");
     qmlRegisterType(componentUrl(QStringLiteral("ApplicationWindow.qml")), uri, 2, 0, "ApplicationWindow");
-    qmlRegisterType(componentUrl(QStringLiteral("BasicListItem.qml")), uri, 2, 0, "BasicListItem");
     qmlRegisterType(componentUrl(QStringLiteral("OverlayDrawer.qml")), uri, 2, 0, "OverlayDrawer");
     qmlRegisterType(componentUrl(QStringLiteral("ContextDrawer.qml")), uri, 2, 0, "ContextDrawer");
     qmlRegisterType(componentUrl(QStringLiteral("GlobalDrawer.qml")), uri, 2, 0, "GlobalDrawer");
@@ -186,29 +143,15 @@ void KirigamiPlugin::registerTypes(const char *uri)
 
     qmlRegisterType<Icon>(uri, 2, 0, "Icon");
 
-    qmlRegisterType(componentUrl(QStringLiteral("Label.qml")), uri, 2, 0, "Label");
-    // TODO: uncomment for 2.3 release
-    // qmlRegisterTypeNotAvailable(uri, 2, 3, "Label", "Label type not supported anymore, use QtQuick.Controls.Label 2.0 instead");
     qmlRegisterType(componentUrl(QStringLiteral("OverlaySheet.qml")), uri, 2, 0, "OverlaySheet");
     qmlRegisterType(componentUrl(QStringLiteral("Page.qml")), uri, 2, 0, "Page");
     qmlRegisterType(componentUrl(QStringLiteral("ScrollablePage.qml")), uri, 2, 0, "ScrollablePage");
-    qmlRegisterType(componentUrl(QStringLiteral("SplitDrawer.qml")), uri, 2, 0, "SplitDrawer");
     qmlRegisterType(componentUrl(QStringLiteral("SwipeListItem.qml")), uri, 2, 0, "SwipeListItem");
 
     // 2.1
-    qmlRegisterType(componentUrl(QStringLiteral("AbstractItemViewHeader.qml")), uri, 2, 1, "AbstractItemViewHeader");
-    qmlRegisterType(componentUrl(QStringLiteral("ItemViewHeader.qml")), uri, 2, 1, "ItemViewHeader");
     qmlRegisterType(componentUrl(QStringLiteral("AbstractApplicationItem.qml")), uri, 2, 1, "AbstractApplicationItem");
     qmlRegisterType(componentUrl(QStringLiteral("ApplicationItem.qml")), uri, 2, 1, "ApplicationItem");
 
-    // 2.2
-    // Theme changed from a singleton to an attached property
-    qmlRegisterUncreatableType<Kirigami::PlatformTheme>(uri,
-                                                        2,
-                                                        2,
-                                                        "Theme",
-                                                        QStringLiteral("Cannot create objects of type Theme, use it as an attached property"));
-
     // 2.3
     qmlRegisterType(componentUrl(QStringLiteral("FormLayout.qml")), uri, 2, 3, "FormLayout");
     qmlRegisterUncreatableType<FormLayoutAttached>(uri,
@@ -229,8 +172,7 @@ void KirigamiPlugin::registerTypes(const char *uri)
     qmlRegisterType(componentUrl(QStringLiteral("CardsGridView.qml")), uri, 2, 4, "CardsGridView");
     qmlRegisterType(componentUrl(QStringLiteral("CardsLayout.qml")), uri, 2, 4, "CardsLayout");
     qmlRegisterType(componentUrl(QStringLiteral("InlineMessage.qml")), uri, 2, 4, "InlineMessage");
-    qmlRegisterUncreatableType<MessageType>(uri, 2, 4, "MessageType", QStringLiteral("Cannot create objects of type MessageType"));
-    qmlRegisterType<DelegateRecycler>(uri, 2, 4, "DelegateRecycler");
+    qmlRegisterUncreatableMetaObject(MessageType::staticMetaObject, uri, 2, 4, "MessageType", QStringLiteral("Cannot create objects of type MessageType"));
 
     // 2.5
     qmlRegisterType(componentUrl(QStringLiteral("ListItemDragHandle.qml")), uri, 2, 5, "ListItemDragHandle");
@@ -266,9 +208,6 @@ void KirigamiPlugin::registerTypes(const char *uri)
     qmlRegisterType<PagePool>(uri, 2, 11, "PagePool");
     qmlRegisterType(componentUrl(QStringLiteral("PagePoolAction.qml")), uri, 2, 11, "PagePoolAction");
 
-    // TODO: remove
-    qmlRegisterType(componentUrl(QStringLiteral("SwipeListItem2.qml")), uri, 2, 11, "SwipeListItem2");
-
     // 2.12
     qmlRegisterType<ShadowedRectangle>(uri, 2, 12, "ShadowedRectangle");
     qmlRegisterType<ShadowedTexture>(uri, 2, 12, "ShadowedTexture");
@@ -280,41 +219,24 @@ void KirigamiPlugin::registerTypes(const char *uri)
     qmlRegisterSingletonType<ColorUtils>(uri, 2, 12, "ColorUtils", singleton<ColorUtils>());
 
     qmlRegisterUncreatableType<CornersGroup>(uri, 2, 12, "CornersGroup", QStringLiteral("Used as grouped property"));
-    qmlRegisterType<PageRouter>(uri, 2, 12, "PageRouter");
-    qmlRegisterType<PageRoute>(uri, 2, 12, "PageRoute");
-    qmlRegisterUncreatableType<PageRouterAttached>(uri, 2, 12, "PageRouterAttached", QStringLiteral("PageRouterAttached cannot be created"));
-    qmlRegisterType(componentUrl(QStringLiteral("RouterWindow.qml")), uri, 2, 12, "RouterWindow");
 
     // 2.13
     qmlRegisterType<ImageColors>(uri, 2, 13, "ImageColors");
-    qmlRegisterType(componentUrl(QStringLiteral("Avatar.qml")), uri, 2, 13, "Avatar");
-    qmlRegisterType(componentUrl(QStringLiteral("swipenavigator/SwipeNavigator.qml")), uri, 2, 13, "SwipeNavigator");
 
     // 2.14
-    qmlRegisterUncreatableType<PreloadRouteGroup>(uri, 2, 14, "PreloadRouteGroup", QStringLiteral("PreloadRouteGroup cannot be created"));
     qmlRegisterType(componentUrl(QStringLiteral("FlexColumn.qml")), uri, 2, 14, "FlexColumn");
     qmlRegisterType<ToolBarLayout>(uri, 2, 14, "ToolBarLayout");
     qmlRegisterSingletonType<DisplayHint>(uri, 2, 14, "DisplayHint", singleton<DisplayHint>());
     qmlRegisterType<SizeGroup>(uri, 2, 14, "SizeGroup");
-    qmlRegisterType<AvatarGroup>("org.kde.kirigami.private", 2, 14, "AvatarGroup");
     qmlRegisterType(componentUrl(QStringLiteral("CheckableListItem.qml")), uri, 2, 14, "CheckableListItem");
-    qmlRegisterSingletonType<NameUtils>(uri, 2, 14, "NameUtils", singleton<NameUtils>());
-
-    qmlRegisterType(componentUrl(QStringLiteral("Hero.qml")), uri, 2, 15, "Hero");
-
-    // 2.16
-    qmlRegisterType<Kirigami::BasicThemeDefinition>(uri, 2, 16, "BasicThemeDefinition");
-
-    // 2.17
-    qmlRegisterType(componentUrl(QStringLiteral("swipenavigator/TabViewLayout.qml")), uri, 2, 17, "TabViewLayout");
-    qmlRegisterType(componentUrl(QStringLiteral("swipenavigator/PageTab.qml")), uri, 2, 17, "PageTab");
 
     // 2.18
-    qmlRegisterType<SpellCheckingAttached>(uri, 2, 18, "SpellChecking");
-    qmlRegisterType(componentUrl(QStringLiteral("settingscomponents/CategorizedSettings.qml")), uri, 2, 18, "CategorizedSettings");
-    qmlRegisterType(componentUrl(QStringLiteral("settingscomponents/GenericSettingsPage.qml")), uri, 2, 18, "GenericSettingsPage");
-    qmlRegisterType(componentUrl(QStringLiteral("settingscomponents/SettingAction.qml")), uri, 2, 18, "SettingAction");
-    
+    qmlRegisterUncreatableType<SpellCheckAttached>(uri,
+                                                   2,
+                                                   18,
+                                                   "SpellCheck",
+                                                   QStringLiteral("Cannot create objects of type SpellCheck, use it as an attached property"));
+
     // 2.19
     qmlRegisterType(componentUrl(QStringLiteral("AboutItem.qml")), uri, 2, 19, "AboutItem");
     qmlRegisterType(componentUrl(QStringLiteral("NavigationTabBar.qml")), uri, 2, 19, "NavigationTabBar");
@@ -322,18 +244,21 @@ void KirigamiPlugin::registerTypes(const char *uri)
     qmlRegisterType(componentUrl(QStringLiteral("Dialog.qml")), uri, 2, 19, "Dialog");
     qmlRegisterType(componentUrl(QStringLiteral("MenuDialog.qml")), uri, 2, 19, "MenuDialog");
     qmlRegisterType(componentUrl(QStringLiteral("PromptDialog.qml")), uri, 2, 19, "PromptDialog");
-    qmlRegisterType(componentUrl(QStringLiteral("AbstractChip.qml")), uri, 2, 19, "AbstractChip");
     qmlRegisterType(componentUrl(QStringLiteral("Chip.qml")), uri, 2, 19, "Chip");
     qmlRegisterType(componentUrl(QStringLiteral("LoadingPlaceholder.qml")), uri, 2, 19, "LoadingPlaceholder");
 
-    qmlRegisterSingletonType<InputMethod>(uri, 2, 19, "InputMethod", [](QQmlEngine *, QJSEngine *) {
-        return new InputMethod{};
-    });
-
     // 2.20
     qmlRegisterType(componentUrl(QStringLiteral("SelectableLabel.qml")), uri, 2, 20, "SelectableLabel");
+    qmlRegisterUncreatableType<OverlayZStackingAttached>(uri,
+                                                         2,
+                                                         20,
+                                                         "OverlayZStacking",
+                                                         QStringLiteral("Cannot create objects of type OverlayZStacking, use it as an attached property"));
+    qmlRegisterType(componentUrl(QStringLiteral("InlineViewHeader.qml")), uri, 2, 20, "InlineViewHeader");
+    qmlRegisterType<Padding>(uri, 2, 20, "Padding");
+
+    qmlRegisterType<HeaderFooterLayout>(uri, 2, 20, "HeaderFooterLayout");
 
-    qmlProtectModule(uri, 2);
 }
 
 void KirigamiPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
@@ -343,24 +268,20 @@ void KirigamiPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
 }
 
 #ifdef KIRIGAMI_BUILD_TYPE_STATIC
-KirigamiPlugin& KirigamiPlugin::getInstance()
+KirigamiPlugin &KirigamiPlugin::getInstance()
 {
     static KirigamiPlugin instance;
     return instance;
 }
 
-void KirigamiPlugin::registerTypes(QQmlEngine* engine)
+void KirigamiPlugin::registerTypes(QQmlEngine *engine)
 {
-    Q_INIT_RESOURCE(shaders);
-    Q_INIT_RESOURCE(KirigamiPlugin);
-
     if (engine) {
         engine->addImportPath(QLatin1String(":/"));
-    }
-    else {
+    } else {
         qCWarning(KirigamiLog)
             << "Registering Kirigami on a null QQmlEngine instance - you likely want to pass a valid engine, or you will want to manually add the "
-            "qrc root path :/ to your import paths list so the engine is able to load the plugin";
+               "qrc root path :/ to your import paths list so the engine is able to load the plugin";
     }
 }
 #endif
