@@ -13,7 +13,6 @@
 #include <QQmlComponent>
 #include <QTimer>
 
-#include "enums.h"
 #include "loggingcategory.h"
 #include "toolbarlayoutdelegate.h"
 
@@ -32,65 +31,68 @@ void ToolBarLayoutAttached::setAction(QObject *action)
     m_action = action;
 }
 
-class ToolBarLayout::Private
+class ToolBarLayoutPrivate
 {
+    ToolBarLayout *const q;
+
 public:
-    Private(ToolBarLayout *qq)
+    ToolBarLayoutPrivate(ToolBarLayout *qq)
         : q(qq)
     {
     }
 
+    void calculateImplicitSize();
     void performLayout();
-    QVector<ToolBarLayoutDelegate *> createDelegates();
+    QList<ToolBarLayoutDelegate *> createDelegates();
     ToolBarLayoutDelegate *createDelegate(QObject *action);
     qreal layoutStart(qreal layoutWidth);
     void maybeHideDelegate(int index, qreal &currentWidth, qreal totalWidth);
 
-    ToolBarLayout *q;
-
-    QVector<QObject *> actions;
-    ActionsProperty actionsProperty;
+    QList<QObject *> actions;
+    ToolBarLayout::ActionsProperty actionsProperty;
     QList<QObject *> hiddenActions;
     QQmlComponent *fullDelegate = nullptr;
     QQmlComponent *iconDelegate = nullptr;
     QQmlComponent *moreButton = nullptr;
     qreal spacing = 0.0;
     Qt::Alignment alignment = Qt::AlignLeft;
+    qreal visibleActionsWidth = 0.0;
     qreal visibleWidth = 0.0;
     Qt::LayoutDirection layoutDirection = Qt::LeftToRight;
-    HeightMode heightMode = ConstrainIfLarger;
+    ToolBarLayout::HeightMode heightMode = ToolBarLayout::ConstrainIfLarger;
 
     bool completed = false;
-    bool layoutQueued = false;
     bool actionsChanged = false;
+    bool implicitSizeValid = false;
+
     std::unordered_map<QObject *, std::unique_ptr<ToolBarLayoutDelegate>> delegates;
-    QVector<ToolBarLayoutDelegate *> sortedDelegates;
+    QList<ToolBarLayoutDelegate *> sortedDelegates;
     QQuickItem *moreButtonInstance = nullptr;
     ToolBarDelegateIncubator *moreButtonIncubator = nullptr;
     bool shouldShowMoreButton = false;
     int firstHiddenIndex = -1;
 
-    QVector<QObject *> removedActions;
+    QList<QObject *> removedActions;
     QTimer *removalTimer = nullptr;
 
     QElapsedTimer performanceTimer;
 
     static void appendAction(ToolBarLayout::ActionsProperty *list, QObject *action);
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    static int actionCount(ToolBarLayout::ActionsProperty *list);
-    static QObject *action(ToolBarLayout::ActionsProperty *list, int index);
-#else
     static qsizetype actionCount(ToolBarLayout::ActionsProperty *list);
     static QObject *action(ToolBarLayout::ActionsProperty *list, qsizetype index);
-#endif
     static void clearActions(ToolBarLayout::ActionsProperty *list);
 };
 
 ToolBarLayout::ToolBarLayout(QQuickItem *parent)
     : QQuickItem(parent)
-    , d(std::make_unique<Private>(this))
+    , d(std::make_unique<ToolBarLayoutPrivate>(this))
 {
-    d->actionsProperty = ActionsProperty(this, this, Private::appendAction, Private::actionCount, Private::action, Private::clearActions);
+    d->actionsProperty = ActionsProperty(this,
+                                         this,
+                                         ToolBarLayoutPrivate::appendAction,
+                                         ToolBarLayoutPrivate::actionCount,
+                                         ToolBarLayoutPrivate::action,
+                                         ToolBarLayoutPrivate::clearActions);
 
     // To prevent multiple assignments to actions from constantly recreating
     // delegates, we cache the delegates and only remove them once they are no
@@ -306,9 +308,8 @@ void ToolBarLayout::setHeightMode(HeightMode newHeightMode)
 
 void ToolBarLayout::relayout()
 {
-    if (d->completed) {
-        polish();
-    }
+    d->implicitSizeValid = false;
+    polish();
 }
 
 void ToolBarLayout::componentComplete()
@@ -318,20 +319,16 @@ void ToolBarLayout::componentComplete()
     relayout();
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-void ToolBarLayout::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
-#else
 void ToolBarLayout::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
-#endif
 {
     if (newGeometry != oldGeometry) {
-        relayout();
+        if (newGeometry.size() != QSizeF{implicitWidth(), implicitHeight()}) {
+            relayout();
+        } else {
+            polish();
+        }
     }
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
-#else
     QQuickItem::geometryChange(newGeometry, oldGeometry);
-#endif
 }
 
 void ToolBarLayout::itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &data)
@@ -347,16 +344,33 @@ void ToolBarLayout::updatePolish()
     d->performLayout();
 }
 
-void ToolBarLayout::Private::performLayout()
+/**
+ * Calculate the implicit size for this layout.
+ *
+ * This is a separate step from performing the actual layout, because of a nasty
+ * little issue with Control, where it will unconditionally set the height of
+ * its contentItem, which means QQuickItem::heightValid() becomes useless. So
+ * instead, we first calculate our implicit size, ignoring any explicitly set
+ * item size. Then we follow that by performing the actual layouting, using the
+ * width and height retrieved from the item, as those will return the explicitly
+ * set width/height if set and the implicit size otherwise. Since control
+ * watches for implicit size changes, we end up with correct behaviour both when
+ * we get an explicit size set and when we're relying on implicit size
+ * calculation.
+ */
+void ToolBarLayoutPrivate::calculateImplicitSize()
 {
+    if (!completed) {
+        return;
+    }
+
     if (!fullDelegate || !iconDelegate || !moreButton) {
         qCWarning(KirigamiLog) << "ToolBarLayout: Unable to layout, required properties are not set";
         return;
     }
 
     if (actions.isEmpty()) {
-        q->setImplicitWidth(0);
-        q->setImplicitHeight(0);
+        q->setImplicitSize(0., 0.);
         return;
     }
 
@@ -403,11 +417,7 @@ void ToolBarLayout::Private::performLayout()
     // The last entry also gets spacing but shouldn't, so remove that.
     maxWidth -= spacing;
 
-    if (q->heightValid() && q->height() > 0.0) {
-        maxHeight = q->height();
-    }
-
-    qreal visibleActionsWidth = 0.0;
+    visibleActionsWidth = 0.0;
 
     if (maxWidth > q->width() - (hiddenActions.isEmpty() ? 0.0 : moreButtonInstance->width() + spacing)) {
         // We have more items than fit into the view, so start hiding some.
@@ -437,24 +447,62 @@ void ToolBarLayout::Private::performLayout()
         visibleActionsWidth = maxWidth;
     }
 
+    if (!hiddenActions.isEmpty()) {
+        maxHeight = std::max(maxHeight, moreButtonInstance->implicitHeight());
+    };
+
+    q->setImplicitSize(maxWidth, maxHeight);
+    Q_EMIT q->hiddenActionsChanged();
+
+    implicitSizeValid = true;
+
+    q->polish();
+}
+
+void ToolBarLayoutPrivate::performLayout()
+{
+    if (!completed || actions.isEmpty()) {
+        return;
+    }
+
+    if (!implicitSizeValid) {
+        calculateImplicitSize();
+    }
+
+    if (sortedDelegates.isEmpty()) {
+        sortedDelegates = createDelegates();
+    }
+
+    bool ready = std::all_of(delegates.cbegin(), delegates.cend(), [](const std::pair<QObject *const, std::unique_ptr<ToolBarLayoutDelegate>> &entry) {
+        return entry.second->isReady();
+    });
+    if (!ready || !moreButtonInstance) {
+        return;
+    }
+
+    qreal width = q->width();
+    qreal height = q->height();
+
     if (!hiddenActions.isEmpty()) {
         if (layoutDirection == Qt::LeftToRight) {
-            moreButtonInstance->setX(q->width() - moreButtonInstance->width());
+            moreButtonInstance->setX(width - moreButtonInstance->width());
         } else {
             moreButtonInstance->setX(0.0);
         }
 
-        if (heightMode == AlwaysFill) {
-            moreButtonInstance->setHeight(q->height());
-        } else if (heightMode == ConstrainIfLarger) {
-            if (moreButtonInstance->implicitHeight() > maxHeight) {
-                moreButtonInstance->setHeight(maxHeight);
+        if (heightMode == ToolBarLayout::AlwaysFill) {
+            moreButtonInstance->setHeight(height);
+        } else if (heightMode == ToolBarLayout::ConstrainIfLarger) {
+            if (moreButtonInstance->implicitHeight() > height) {
+                moreButtonInstance->setHeight(height);
             } else {
-                moreButtonInstance->setHeight(moreButtonInstance->implicitHeight());
+                moreButtonInstance->resetHeight();
             }
+        } else {
+            moreButtonInstance->resetHeight();
         }
 
-        moreButtonInstance->setY(qRound((maxHeight - moreButtonInstance->height()) / 2.0));
+        moreButtonInstance->setY(qRound((height - moreButtonInstance->height()) / 2.0));
         shouldShowMoreButton = true;
         moreButtonInstance->setVisible(true);
     } else {
@@ -462,27 +510,25 @@ void ToolBarLayout::Private::performLayout()
         moreButtonInstance->setVisible(false);
     }
 
-    if (moreButtonInstance->isVisible() && !q->heightValid()) {
-        maxHeight = std::max(maxHeight, moreButtonInstance->implicitHeight());
-    };
-
     qreal currentX = layoutStart(visibleActionsWidth);
     for (auto entry : std::as_const(sortedDelegates)) {
         if (!entry->isVisible()) {
             continue;
         }
 
-        if (heightMode == AlwaysFill) {
-            entry->setHeight(q->height());
-        } else if (heightMode == ConstrainIfLarger) {
-            if (entry->implicitHeight() > maxHeight) {
-                entry->setHeight(maxHeight);
+        if (heightMode == ToolBarLayout::AlwaysFill) {
+            entry->setHeight(height);
+        } else if (heightMode == ToolBarLayout::ConstrainIfLarger) {
+            if (entry->implicitHeight() > height) {
+                entry->setHeight(height);
             } else {
-                entry->setHeight(entry->implicitHeight());
+                entry->resetHeight();
             }
+        } else {
+            entry->resetHeight();
         }
 
-        qreal y = qRound((maxHeight - entry->height()) / 2.0);
+        qreal y = qRound((height - entry->height()) / 2.0);
 
         if (layoutDirection == Qt::LeftToRight) {
             entry->setPosition(currentX, y);
@@ -495,9 +541,6 @@ void ToolBarLayout::Private::performLayout()
         entry->show();
     }
 
-    q->setImplicitSize(maxWidth, maxHeight);
-    Q_EMIT q->hiddenActionsChanged();
-
     qreal newVisibleWidth = visibleActionsWidth;
     if (moreButtonInstance->isVisible()) {
         newVisibleWidth += moreButtonInstance->width() + (newVisibleWidth > 0.0 ? spacing : 0.0);
@@ -518,9 +561,9 @@ void ToolBarLayout::Private::performLayout()
     sortedDelegates.clear();
 }
 
-QVector<ToolBarLayoutDelegate *> ToolBarLayout::Private::createDelegates()
+QList<ToolBarLayoutDelegate *> ToolBarLayoutPrivate::createDelegates()
 {
-    QVector<ToolBarLayoutDelegate *> result;
+    QList<ToolBarLayoutDelegate *> result;
     for (auto action : std::as_const(actions)) {
         if (delegates.find(action) != delegates.end()) {
             result.append(delegates.at(action).get());
@@ -542,12 +585,10 @@ QVector<ToolBarLayoutDelegate *> ToolBarLayout::Private::createDelegates()
             moreButtonInstance = qobject_cast<QQuickItem *>(incubator->object());
             moreButtonInstance->setVisible(false);
 
-            connect(moreButtonInstance, &QQuickItem::visibleChanged, q, [this]() {
+            QObject::connect(moreButtonInstance, &QQuickItem::visibleChanged, q, [this]() {
                 moreButtonInstance->setVisible(shouldShowMoreButton);
             });
-            connect(moreButtonInstance, &QQuickItem::widthChanged, q, [this]() {
-                Q_EMIT q->minimumWidthChanged();
-            });
+            QObject::connect(moreButtonInstance, &QQuickItem::widthChanged, q, &ToolBarLayout::minimumWidthChanged);
             q->relayout();
             Q_EMIT q->minimumWidthChanged();
 
@@ -562,7 +603,7 @@ QVector<ToolBarLayoutDelegate *> ToolBarLayout::Private::createDelegates()
     return result;
 }
 
-ToolBarLayoutDelegate *ToolBarLayout::Private::createDelegate(QObject *action)
+ToolBarLayoutDelegate *ToolBarLayoutPrivate::createDelegate(QObject *action)
 {
     QQmlComponent *fullComponent = nullptr;
     auto displayComponent = action->property("displayComponent");
@@ -585,7 +626,7 @@ ToolBarLayoutDelegate *ToolBarLayout::Private::createDelegate(QObject *action)
     return result;
 }
 
-qreal ToolBarLayout::Private::layoutStart(qreal layoutWidth)
+qreal ToolBarLayoutPrivate::layoutStart(qreal layoutWidth)
 {
     qreal availableWidth = moreButtonInstance->isVisible() ? q->width() - (moreButtonInstance->width() + spacing) : q->width();
 
@@ -600,7 +641,7 @@ qreal ToolBarLayout::Private::layoutStart(qreal layoutWidth)
     return 0.0;
 }
 
-void ToolBarLayout::Private::maybeHideDelegate(int index, qreal &currentWidth, qreal totalWidth)
+void ToolBarLayoutPrivate::maybeHideDelegate(int index, qreal &currentWidth, qreal totalWidth)
 {
     auto delegate = sortedDelegates.at(index);
 
@@ -690,31 +731,25 @@ void ToolBarLayout::Private::maybeHideDelegate(int index, qreal &currentWidth, q
     }
 }
 
-void ToolBarLayout::Private::appendAction(ToolBarLayout::ActionsProperty *list, QObject *action)
+void ToolBarLayoutPrivate::appendAction(ToolBarLayout::ActionsProperty *list, QObject *action)
 {
     auto layout = reinterpret_cast<ToolBarLayout *>(list->data);
     layout->addAction(action);
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-int ToolBarLayout::Private::actionCount(ToolBarLayout::ActionsProperty *list)
-#else
-qsizetype ToolBarLayout::Private::actionCount(ToolBarLayout::ActionsProperty *list)
-#endif
+qsizetype ToolBarLayoutPrivate::actionCount(ToolBarLayout::ActionsProperty *list)
 {
     return reinterpret_cast<ToolBarLayout *>(list->data)->d->actions.count();
 }
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-QObject *ToolBarLayout::Private::action(ToolBarLayout::ActionsProperty *list, int index)
-#else
-QObject *ToolBarLayout::Private::action(ToolBarLayout::ActionsProperty *list, qsizetype index)
-#endif
+QObject *ToolBarLayoutPrivate::action(ToolBarLayout::ActionsProperty *list, qsizetype index)
 {
     return reinterpret_cast<ToolBarLayout *>(list->data)->d->actions.at(index);
 }
 
-void ToolBarLayout::Private::clearActions(ToolBarLayout::ActionsProperty *list)
+void ToolBarLayoutPrivate::clearActions(ToolBarLayout::ActionsProperty *list)
 {
     reinterpret_cast<ToolBarLayout *>(list->data)->clearActions();
 }
+
+#include "moc_toolbarlayout.cpp"
