@@ -10,16 +10,18 @@
 #include <QQuickItem>
 #include <memory>
 
+class ToolBarLayoutPrivate;
+
 /**
- * @brief Attached property for ToolBarLayout delegates.
+ * Attached property for ToolBarLayout delegates.
  */
 class ToolBarLayoutAttached : public QObject
 {
     Q_OBJECT
     /**
-     * @brief This property holds the action that this delegate was created for.
+     * The action this delegate was created for.
      */
-    Q_PROPERTY(QObject *action READ action CONSTANT)
+    Q_PROPERTY(QObject *action READ action CONSTANT FINAL)
 public:
     ToolBarLayoutAttached(QObject *parent = nullptr);
 
@@ -31,14 +33,13 @@ private:
 };
 
 /**
- * @brief This is a layout that creates delegates for actions and lays them out in a row.
+ * An item that creates delegates for actions and lays them out in a row.
  *
  * This effectively combines RowLayout and Repeater in a single item, with the
  * addition of some extra performance enhancing tweaks. It will create instances
- * of ::fullDelegate and ::iconDelegate for each action in ::actions. These are
+ * of ::fullDelegate and ::itemDelegate for each action in ::actions . These are
  * then positioned horizontally. Any action that ends up being placed outside
- * the width of the item is hidden and will be part of ::hiddenActions which are
- * shown in a overflow menu.
+ * the width of the item is hidden and will be part of ::hiddenActions.
  *
  * The items created as delegates are always created asynchronously, to avoid
  * creation lag spikes. Each delegate has access to the action it was created
@@ -48,73 +49,63 @@ class ToolBarLayout : public QQuickItem
 {
     Q_OBJECT
     /**
-     * @brief This property holds a list of visible actions this layout should create delegates for.
+     * The actions this layout should create delegates for.
      */
-    Q_PROPERTY(QQmlListProperty<QObject> actions READ actionsProperty NOTIFY actionsChanged)
+    Q_PROPERTY(QQmlListProperty<QObject> actions READ actionsProperty NOTIFY actionsChanged FINAL)
     /**
-     * @brief This property holds a list of hidden actions that do not fit in the current view,
-     * and are thus hidden.
+     * A list of actions that do not fit in the current view and are thus hidden.
      */
-    Q_PROPERTY(QList<QObject *> hiddenActions READ hiddenActions NOTIFY hiddenActionsChanged)
+    Q_PROPERTY(QList<QObject *> hiddenActions READ hiddenActions NOTIFY hiddenActionsChanged FINAL)
     /**
-     * @brief This property holds a component that is used to create full-size delegates.
+     * A component that is used to create full-size delegates from.
      *
      * Each delegate has three states, it can be full-size, icon-only or hidden.
-     * By default, the full-size delegate is used.
-     *
-     * The full-size delegate is used when the action has its display hint set to
-     * @link DisplayHint.KeepVisible KeepVisible @endlink. If there is not enough
-     * space, it will either use iconDelegate or the delegate will be shown in the
-     * overflow menu.
-     *
-     * @see kirigami::Action::displayComponent
+     * By default, the full-size delegate is used. When the action has the
+     * DisplayHint::IconOnly hint set, it will always use the iconDelegate. When
+     * it has the DisplayHint::KeepVisible hint set, it will use the full-size
+     * delegate when it fits. If not, it will use the iconDelegate, unless even
+     * that does not fit, in which case it will still be hidden.
      */
-    Q_PROPERTY(QQmlComponent *fullDelegate READ fullDelegate WRITE setFullDelegate NOTIFY fullDelegateChanged)
+    Q_PROPERTY(QQmlComponent *fullDelegate READ fullDelegate WRITE setFullDelegate NOTIFY fullDelegateChanged FINAL)
     /**
-     * @brief This property holds a component that is used to create icon-only delegates.
+     * A component that is used to create icon-only delegates from.
      *
-     * This is used when display hint is set to @link DisplayHint.IconOnly IconOnly @endlink,
-     * unless there is not enough space, in which case it will show the delegate in the
-     * overflow menu.
-     *
-     * @see ::fullDelegate
+     * \sa fullDelegate
      */
-    Q_PROPERTY(QQmlComponent *iconDelegate READ iconDelegate WRITE setIconDelegate NOTIFY iconDelegateChanged)
+    Q_PROPERTY(QQmlComponent *iconDelegate READ iconDelegate WRITE setIconDelegate NOTIFY iconDelegateChanged FINAL)
     /**
-     * @brief This property holds a component that is used to create the "more button" item from.
+     * A component that is used to create the "more button" item from.
      *
      * The more button is shown when there are actions that do not fit the
      * current view. It is intended to have functionality to show these hidden
      * actions, like popup a menu with them showing.
      */
-    Q_PROPERTY(QQmlComponent *moreButton READ moreButton WRITE setMoreButton NOTIFY moreButtonChanged)
+    Q_PROPERTY(QQmlComponent *moreButton READ moreButton WRITE setMoreButton NOTIFY moreButtonChanged FINAL)
     /**
-     * @brief This property holds the amount of spacing between individual delegates.
+     * The amount of spacing between individual delegates.
      */
-    Q_PROPERTY(qreal spacing READ spacing WRITE setSpacing NOTIFY spacingChanged)
+    Q_PROPERTY(qreal spacing READ spacing WRITE setSpacing NOTIFY spacingChanged FINAL)
     /**
-     * @brief This property sets how to align the delegates within this layout.
+     * How to align the delegates within this layout.
      *
      * When there is more space available than required by the visible delegates,
      * we need to determine how to place the delegates. This property determines
-     * how to do that.
-     *
-     * @note The moreButton, if visible, will always be placed at the end of the layout.
-     * @see <a href="https://doc.qt.io/qt-6/qt.html#Alignment">Qt.Alignment</a>
+     * how to do that. Note that the moreButton, if visible, will always be
+     * placed at the end of the layout.
      */
-    Q_PROPERTY(Qt::Alignment alignment READ alignment WRITE setAlignment NOTIFY alignmentChanged)
+    Q_PROPERTY(Qt::Alignment alignment READ alignment WRITE setAlignment NOTIFY alignmentChanged FINAL)
     /**
-     * @brief This property holds the combined width of visible delegates in this layout.
+     * The combined width of visible delegates in this layout.
      */
-    Q_PROPERTY(qreal visibleWidth READ visibleWidth NOTIFY visibleWidthChanged)
+    Q_PROPERTY(qreal visibleWidth READ visibleWidth NOTIFY visibleWidthChanged FINAL)
     /**
-     * @brief This property holds the minimum width this layout can have.
+     * The minimum width this layout can have.
      *
      * This is equal to the width of the moreButton.
      */
-    Q_PROPERTY(qreal minimumWidth READ minimumWidth NOTIFY minimumWidthChanged)
+    Q_PROPERTY(qreal minimumWidth READ minimumWidth NOTIFY minimumWidthChanged FINAL)
     /**
-     * @brief This property sets the layout direction of the toolbar layout.
+     * Which direction to layout in.
      *
      * This is primarily intended to support right-to-left layouts. When set to
      * LeftToRight, delegates will be layout with the first item on the left and
@@ -126,41 +117,30 @@ class ToolBarLayout : public QQuickItem
      * be placed at the leftmost position. Alignment flags are inverted, so
      * AlignLeft will align items to the right, and vice-versa.
      */
-    Q_PROPERTY(Qt::LayoutDirection layoutDirection READ layoutDirection WRITE setLayoutDirection NOTIFY layoutDirectionChanged)
+    Q_PROPERTY(Qt::LayoutDirection layoutDirection READ layoutDirection WRITE setLayoutDirection NOTIFY layoutDirectionChanged FINAL)
     /**
-     * @brief This property sets how to handle items that do not match the toolbar's height.
+     * How to handle items that do not match the toolbar's height.
      *
      * When toolbar items do not match the height of the toolbar, there are
      * several ways we can deal with this. This property sets the preferred way.
      *
-     * default: ``ToolBarLayout.HeightMode.ConstrainIfLarger``
+     * The default is HeightMode::ConstrainIfLarger .
      *
-     * @see ::HeightMode
+     * \sa HeightMode
      */
-    Q_PROPERTY(HeightMode heightMode READ heightMode WRITE setHeightMode NOTIFY heightModeChanged)
+    Q_PROPERTY(HeightMode heightMode READ heightMode WRITE setHeightMode NOTIFY heightModeChanged FINAL)
 
 public:
     using ActionsProperty = QQmlListProperty<QObject>;
 
     /**
-     * @brief This enum describes several modes that can be used to deal with items with
-     * height that does not match the toolbar's height.
+     * An enum describing several modes that can be used to deal with items with
+     * a height that does not match the toolbar's height.
      */
     enum HeightMode {
-        /**
-         * @brief Always center items, allowing them to go outside the bounds of the layout if they are larger.
-         */
-        AlwaysCenter,
-
-        /**
-         * @brief Always match the height of the layout. Larger items will be reduced in height, smaller items will be increased.
-         */
-        AlwaysFill,
-
-        /**
-         * @brief If the item is larger than the toolbar, reduce its height. Otherwise center it in the toolbar.
-         */
-        ConstrainIfLarger,
+        AlwaysCenter, ///< Always center items, allowing them to go outside the bounds of the layout if they are larger.
+        AlwaysFill, ///< Always match the height of the layout. Larger items will be reduced in height, smaller items will be increased.
+        ConstrainIfLarger, ///< If the item is larger than the toolbar, reduce its height. Otherwise center it in the toolbar.
     };
     Q_ENUM(HeightMode)
 
@@ -169,19 +149,19 @@ public:
 
     ActionsProperty actionsProperty() const;
     /**
-     * @brief This method adds an action to the list of actions.
+     * Add an action to the list of actions.
      *
-     * @param action The action to add.
+     * \param action The action to add.
      */
     void addAction(QObject *action);
     /**
-     * This method  removes an action from the list of actions.
+     * Remove an action from the list of actions.
      *
-     * @param action The action to remove.
+     * \param action The action to remove.
      */
     void removeAction(QObject *action);
     /**
-     * @brief This method clears the list of actions.
+     * Clear the list of actions.
      */
     void clearActions();
     Q_SIGNAL void actionsChanged();
@@ -224,9 +204,9 @@ public:
     Q_SIGNAL void heightModeChanged();
 
     /**
-     * @brief This slot queues a relayout of this ToolBarLayout.
+     * Queue a relayout of this layout.
      *
-     * @note The layouting happens during the next scene graph polishing phase.
+     * \note The layouting happens during the next scene graph polishing phase.
      */
     Q_SLOT void relayout();
 
@@ -237,17 +217,13 @@ public:
 
 protected:
     void componentComplete() override;
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    void geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry) override;
-#else
     void geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry) override;
-#endif
     void itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &data) override;
     void updatePolish() override;
 
 private:
-    class Private;
-    const std::unique_ptr<Private> d;
+    friend class ToolBarLayoutPrivate;
+    const std::unique_ptr<ToolBarLayoutPrivate> d;
 };
 
 QML_DECLARE_TYPEINFO(ToolBarLayout, QML_HAS_ATTACHED_PROPERTIES)
