@@ -8,11 +8,14 @@
 #include <QGuiApplication>
 #include <QObject>
 #include <QPoint>
+#include <QPropertyAnimation>
+#include <QQmlParserStatus>
 #include <QQuickItem>
 #include <QStyleHints>
 #include <QTimer>
 
 class QWheelEvent;
+class QQmlEngine;
 class WheelHandler;
 
 /**
@@ -23,62 +26,68 @@ class KirigamiWheelEvent : public QObject
     Q_OBJECT
 
     /**
-     * @brief This property holds the x coordinate of the mouse pointer.
-     */
-    Q_PROPERTY(qreal x READ x CONSTANT)
-
-    /**
-     * @brief This property holds the y coordinate of the mouse pointer.
-     */
-    Q_PROPERTY(qreal y READ y CONSTANT)
-
-    /**
-     * @brief This property holds the distance the wheel is rotated in degrees.
+     * x: real
      *
+     * X coordinate of the mouse pointer
+     */
+    Q_PROPERTY(qreal x READ x CONSTANT FINAL)
+
+    /**
+     * y: real
+     *
+     * Y coordinate of the mouse pointer
+     */
+    Q_PROPERTY(qreal y READ y CONSTANT FINAL)
+
+    /**
+     * angleDelta: point
+     *
+     * The distance the wheel is rotated in degrees.
      * The x and y coordinates indicate the horizontal and vertical wheels respectively.
      * A positive value indicates it was rotated up/right, negative, bottom/left
      * This value is more likely to be set in traditional mice.
      */
-    Q_PROPERTY(QPointF angleDelta READ angleDelta CONSTANT)
+    Q_PROPERTY(QPointF angleDelta READ angleDelta CONSTANT FINAL)
 
     /**
-     * @brief This property provides the delta in screen pixels available on high resolution trackpads.
-     */
-    Q_PROPERTY(QPointF pixelDelta READ pixelDelta CONSTANT)
-
-    /**
-     * @brief This property contains an OR combination of the buttons that were pressed during the wheel.
+     * pixelDelta: point
      *
-     * Possible values in a combination are:
-     * * ``Qt.LeftButton``
-     * * ``Qt.MiddleButton``
-     * * ``Qt.RightButton``
+     * provides the delta in screen pixels available on high resolution trackpads
      */
-    Q_PROPERTY(int buttons READ buttons CONSTANT)
+    Q_PROPERTY(QPointF pixelDelta READ pixelDelta CONSTANT FINAL)
 
     /**
-     * @brief This property holds the keyboard modifiers that were pressed during the wheel event.
+     * buttons: int
      *
-     * Possible values in a combination are:
-     * * ``Qt.NoModifier`` (default, no modifiers)
-     * * ``Qt.ControlModifier``
-     * * ``Qt.ShiftModifier``
+     * it contains an OR combination of the buttons that were pressed during the wheel, they can be:
+     * Qt.LeftButton, Qt.MiddleButton, Qt.RightButton
+     */
+    Q_PROPERTY(int buttons READ buttons CONSTANT FINAL)
+
+    /**
+     * modifiers: int
+     *
+     * Keyboard mobifiers that were pressed during the wheel event, such as:
+     * Qt.NoModifier (default, no modifiers)
+     * Qt.ControlModifier
+     * Qt.ShiftModifier
      * ...
      */
-    Q_PROPERTY(int modifiers READ modifiers CONSTANT)
+    Q_PROPERTY(int modifiers READ modifiers CONSTANT FINAL)
 
     /**
-     * @brief This property holds whether the delta values are inverted.
+     * inverted: bool
      *
-     * The returned delta may be inverted on some platforms, so positive values would mean bottom/left.
+     * Whether the delta values are inverted
+     * On some platformsthe returned delta are inverted, so positive values would mean bottom/left
      */
-    Q_PROPERTY(bool inverted READ inverted CONSTANT)
+    Q_PROPERTY(bool inverted READ inverted CONSTANT FINAL)
 
     /**
-     * @brief This property sets whether the event should be accepted or dropped.
-     * If set, the event shouldn't be managed anymore, for instance it can be used
-     * to block the handler to manage the scroll of a view on some scenarios.
+     * accepted: bool
      *
+     * If set, the event shouldn't be managed anymore,
+     * for instance it can be used to block the handler to manage the scroll of a view on some scenarios
      * @code
      * // This handler handles automatically the scroll of
      * // flickableItem, unless Ctrl is pressed, in this case the
@@ -97,7 +106,7 @@ class KirigamiWheelEvent : public QObject
      * @endcode
      *
      */
-    Q_PROPERTY(bool accepted READ isAccepted WRITE setAccepted)
+    Q_PROPERTY(bool accepted READ isAccepted WRITE setAccepted FINAL)
 
 public:
     KirigamiWheelEvent(QObject *parent = nullptr);
@@ -136,21 +145,18 @@ public:
 /**
  * @brief Handles scrolling for a Flickable and 2 attached ScrollBars.
  *
- * WheelHandler filters events from a QtQuick.Flickable,
- * a vertical QtQuick.Controls.ScrollBar and a horizontal QtQuick.Controls.ScrollBar.
- * Wheel and KeyPress events (when ::keyNavigationEnabled is true) are
- * used to scroll the Flickable. When ::filterMouseEvents is true, WheelHandler blocks mouse button
- * input from reaching the Flickable and sets the
- * <a href="https://doc.qt.io/qt-5/qml-qtquick-controls2-scrollbar.html#interactive-prop">interactive</a>
- * property of the scrollbars to @c false when touch input is used.
+ * WheelHandler filters events from a Flickable, a vertical ScrollBar and a horizontal ScrollBar.
+ * Wheel and KeyPress events (when `keyNavigationEnabled` is true) are used to scroll the Flickable.
+ * When `filterMouseEvents` is true, WheelHandler blocks mouse button input from reaching the Flickable
+ * and sets the `interactive` property of the scrollbars to false when touch input is used.
  *
  * Wheel event handling behavior:
  *
  * - Pixel delta is ignored unless angle delta is not available because pixel delta scrolling is too slow. Qt Widgets doesn't use pixel delta either, so the
  * default scroll speed should be consistent with Qt Widgets.
- * - When using angle delta, scroll using the step increments defined by ::verticalStepSize and ::horizontalStepSize.
- * - When one of the keyboard modifiers in ::pageScrollModifiers is used, scroll by pages.
- * - When using a device that doesn't use 120 angle delta unit increments such as a touchpad, the ::verticalStepSize, ::horizontalStepSize and page increments
+ * - When using angle delta, scroll using the step increments defined by `verticalStepSize` and `horizontalStepSize`.
+ * - When one of the keyboard modifiers in `pageScrollModifiers` is used, scroll by pages.
+ * - When using a device that doesn't use 120 angle delta unit increments such as a touchpad, the `verticalStepSize`, `horizontalStepSize` and page increments
  * (if using page scrolling) will be multiplied by `angle delta / 120` to keep scrolling smooth.
  * - If scrolling has happened in the last 400ms, use an internal QQuickItem stacked over the Flickable's contentItem to catch wheel events and use those wheel
  * events to scroll, if possible. This prevents controls inside the Flickable's contentItem that allow scrolling to change the value (e.g., Sliders, SpinBoxes)
@@ -165,55 +171,51 @@ public:
  * @include wheelhandler/ScrollViewUsage.qml
  *
  */
-class WheelHandler : public QObject
+class WheelHandler : public QObject, public QQmlParserStatus
 {
     Q_OBJECT
+    Q_INTERFACES(QQmlParserStatus)
 
     /**
-     * @brief This property holds the Flickable that the WheelHandler will control.
+     * @brief This property holds the Qt Quick Flickable that the WheelHandler will control.
      */
     Q_PROPERTY(QQuickItem *target READ target WRITE setTarget NOTIFY targetChanged FINAL)
 
     /**
      * @brief This property holds the vertical step size.
      *
-     * The default value is equivalent to `20 * Qt.styleHints.wheelScrollLines`.
-     * This is consistent with the default increment for QScrollArea.
+     * The default value is equivalent to `20 * Qt.styleHints.wheelScrollLines`. This is consistent with the default increment for QScrollArea.
+     *
+     * @sa horizontalStepSize
      *
-     * @see ::horizontalStepSize
      * @since KDE Frameworks 5.89
      */
-    Q_PROPERTY(qreal verticalStepSize READ verticalStepSize
-               WRITE setVerticalStepSize RESET resetVerticalStepSize
-               NOTIFY verticalStepSizeChanged FINAL)
+    Q_PROPERTY(qreal verticalStepSize READ verticalStepSize WRITE setVerticalStepSize RESET resetVerticalStepSize NOTIFY verticalStepSizeChanged FINAL)
 
     /**
      * @brief This property holds the horizontal step size.
      *
-     * The default value is equivalent to `20 * Qt.styleHints.wheelScrollLines`.
-     * This is consistent with the default increment for QScrollArea.
+     * The default value is equivalent to `20 * Qt.styleHints.wheelScrollLines`. This is consistent with the default increment for QScrollArea.
+     *
+     * @sa verticalStepSize
      *
-     * @see ::verticalStepSize
      * @since KDE Frameworks 5.89
      */
-    Q_PROPERTY(qreal horizontalStepSize READ horizontalStepSize
-               WRITE setHorizontalStepSize RESET resetHorizontalStepSize
-               NOTIFY horizontalStepSizeChanged FINAL)
+    Q_PROPERTY(
+        qreal horizontalStepSize READ horizontalStepSize WRITE setHorizontalStepSize RESET resetHorizontalStepSize NOTIFY horizontalStepSizeChanged FINAL)
 
     /**
      * @brief This property holds the keyboard modifiers that will be used to start page scrolling.
      *
-     * The default value is equivalent to `Qt.ControlModifier | Qt.ShiftModifier`. This matches QScrollBar,
-     * which uses QAbstractSlider behavior.
+     * The default value is equivalent to `Qt.ControlModifier | Qt.ShiftModifier`. This matches QScrollBar, which uses QAbstractSlider behavior.
      *
      * @since KDE Frameworks 5.89
      */
-    Q_PROPERTY(Qt::KeyboardModifiers pageScrollModifiers READ pageScrollModifiers
-               WRITE setPageScrollModifiers RESET resetPageScrollModifiers
-               NOTIFY pageScrollModifiersChanged FINAL)
+    Q_PROPERTY(Qt::KeyboardModifiers pageScrollModifiers READ pageScrollModifiers WRITE setPageScrollModifiers RESET resetPageScrollModifiers NOTIFY
+                   pageScrollModifiersChanged FINAL)
 
     /**
-     * @brief This property holds whether the WheelHandler filters mouse events like a QtQuick.Controls.ScrollView would.
+     * @brief This property holds whether the WheelHandler filters mouse events like a Qt Quick Controls ScrollView would.
      *
      * Touch events are allowed to flick the view and they make the scrollbars not interactive.
      *
@@ -221,12 +223,11 @@ class WheelHandler : public QObject
      *
      * Hover events on the scrollbars and wheel events on anything also make the scrollbars interactive when this property is set to true.
      *
-     * default: ``false``
+     * The default value is `false`.
      *
      * @since KDE Frameworks 5.89
      */
-    Q_PROPERTY(bool filterMouseEvents READ filterMouseEvents
-               WRITE setFilterMouseEvents NOTIFY filterMouseEventsChanged FINAL)
+    Q_PROPERTY(bool filterMouseEvents READ filterMouseEvents WRITE setFilterMouseEvents NOTIFY filterMouseEventsChanged FINAL)
 
     /**
      * @brief This property holds whether the WheelHandler handles keyboard scrolling.
@@ -241,12 +242,11 @@ class WheelHandler : public QObject
      * - End scrolls to the end.
      * - When Alt is held, scroll horizontally when using PageUp, PageDown, Home or End.
      *
-     * default: ``false``
+     * The default value is `false`.
      *
      * @since KDE Frameworks 5.89
      */
-    Q_PROPERTY(bool keyNavigationEnabled READ keyNavigationEnabled
-               WRITE setKeyNavigationEnabled NOTIFY keyNavigationEnabledChanged FINAL)
+    Q_PROPERTY(bool keyNavigationEnabled READ keyNavigationEnabled WRITE setKeyNavigationEnabled NOTIFY keyNavigationEnabledChanged FINAL)
 
     /**
      * @brief This property holds whether the WheelHandler blocks all wheel events from reaching the Flickable.
@@ -257,16 +257,16 @@ class WheelHandler : public QObject
      * NOTE: Wheel events created by touchpad gestures with pixel deltas will always be accepted no matter what. This is because they will cause the Flickable
      * to jump back to where scrolling started unless the events are always accepted before they reach the Flickable.
      *
-     * default: ``true``
+     * The default value is true.
      */
-    Q_PROPERTY(bool blockTargetWheel MEMBER m_blockTargetWheel NOTIFY blockTargetWheelChanged)
+    Q_PROPERTY(bool blockTargetWheel MEMBER m_blockTargetWheel NOTIFY blockTargetWheelChanged FINAL)
 
     /**
      * @brief This property holds whether the WheelHandler can use wheel events to scroll the Flickable.
      *
-     * default: ``true``
+     * The default value is true.
      */
-    Q_PROPERTY(bool scrollFlickableTarget MEMBER m_scrollFlickableTarget NOTIFY scrollFlickableTargetChanged)
+    Q_PROPERTY(bool scrollFlickableTarget MEMBER m_scrollFlickableTarget NOTIFY scrollFlickableTargetChanged FINAL)
 
 public:
     explicit WheelHandler(QObject *parent = nullptr);
@@ -294,36 +294,36 @@ public:
     void setKeyNavigationEnabled(bool enabled);
 
     /**
-     * Scroll up one step. If the ::stepSize parameter is less than 0, the ::verticalStepSize will be used.
+     * Scroll up one step. If the stepSize parameter is less than 0, the verticalStepSize will be used.
      *
-     * returns @c true if the contentItem was moved.
+     * returns true if the contentItem was moved.
      *
      * @since KDE Frameworks 5.89
      */
     Q_INVOKABLE bool scrollUp(qreal stepSize = -1);
 
     /**
-     * Scroll down one step. If the ::stepSize parameter is less than 0, the ::verticalStepSize will be used.
+     * Scroll down one step. If the stepSize parameter is less than 0, the verticalStepSize will be used.
      *
-     * returns @c true if the contentItem was moved.
+     * returns true if the contentItem was moved.
      *
      * @since KDE Frameworks 5.89
      */
     Q_INVOKABLE bool scrollDown(qreal stepSize = -1);
 
     /**
-     * Scroll left one step. If the ::stepSize parameter is less than 0, the ::horizontalStepSize will be used.
+     * Scroll left one step. If the stepSize parameter is less than 0, the horizontalStepSize will be used.
      *
-     * returns @c true if the contentItem was moved.
+     * returns true if the contentItem was moved.
      *
      * @since KDE Frameworks 5.89
      */
     Q_INVOKABLE bool scrollLeft(qreal stepSize = -1);
 
     /**
-     * Scroll right one step. If the ::stepSize parameter is less than 0, the ::horizontalStepSize will be used.
+     * Scroll right one step. If the stepSize parameter is less than 0, the horizontalStepSize will be used.
      *
-     * returns @c true if the contentItem was moved.
+     * returns true if the contentItem was moved.
      *
      * @since KDE Frameworks 5.89
      */
@@ -353,10 +353,11 @@ private Q_SLOTS:
     void _k_rebindScrollBars();
 
 private:
+    void classBegin() override;
+    void componentComplete() override;
+
     void setScrolling(bool scrolling);
-    bool scrollFlickable(QPointF pixelDelta,
-                         QPointF angleDelta = {},
-                         Qt::KeyboardModifiers modifiers = Qt::NoModifier);
+    bool scrollFlickable(QPointF pixelDelta, QPointF angleDelta = {}, Qt::KeyboardModifiers modifiers = Qt::NoModifier);
 
     QPointer<QQuickItem> m_flickable;
     QPointer<QQuickItem> m_verticalScrollBar;
@@ -374,7 +375,6 @@ private:
     constexpr static qreal m_wheelScrollingDuration = 400;
     bool m_filterMouseEvents = false;
     bool m_keyNavigationEnabled = false;
-    bool m_wasTouched = false;
     bool m_blockTargetWheel = true;
     bool m_scrollFlickableTarget = true;
     // Same as QXcbWindow.
@@ -384,4 +384,9 @@ private:
     Qt::KeyboardModifiers m_pageScrollModifiers = m_defaultPageScrollModifiers;
     QTimer m_wheelScrollingTimer;
     KirigamiWheelEvent m_kirigamiWheelEvent;
+
+    // Smooth scrolling
+    QQmlEngine *m_engine = nullptr;
+    QPropertyAnimation m_yScrollAnimation{nullptr, "contentY"};
+    bool m_wasTouched = false;
 };
